/* © brix cross media AG, http://www.brix.ch - It is strictly forbidden to change, reuse or copy the source code of this document. */
;(function ( $, window, document, undefined ) {
    var pluginName = "anuraMasonry",
        defaults = {
            server: 'https://celum.local/anura/first', // REQUIRED, which anura endpoint to use
            ext: '.do?', // extension of server pages (.php, .do)
            node: {kind: 101, id: 0}, // which node of what kind to show (101 = folder, 102 = collection, 103 = keyword)
            top: {number: 0, days: 0}, // show the top X downloads within the last Y days
            relations: {id: 0, version: null, type: null}, // show related assets of a particular asset
            assets: null, // load a defined list of assets from somewhere else
            search: {string: '', node: 0, custom: ''},
            sort: null, // order: one of [id, name, uploaddate, lastmodified, extension, assettype], seq: asc or desc - i.e. {order: 'name', seq: 'asc'}
            locale: '', // ISO 639-1 code, blank for default
            paginate: 'auto', // how many items to show per page, false to show all, 'auto' to load on scroll
            paginator: ['prev', 'next', 'pages', 'counter'],
            paginator_nav: {prev: '&larr;', next: '&rarr;'},
            autoload_fetch: 50, // how many assets to fetch at once when paginage is set to 'auto'
            limit: 0, // limit the results (or reported total) returned by the server
            detail_view: 'anura', // one of: slimbox, colorbox, details, blank, none
            detail_settings: {}, // settings to pass to anura.details.js
            colorbox_min_height: 500, colorbox_settings: {}, // colorbox specific settings since detail_settings is used by anuraDetails
            download_menu: true, // show the download menu
            download_directly: false, // make the different formats available directly in the menu
            force_zip: false, // force even single asset downloads to be delivered in a ZIP-file
            icon_transform: function(icon) {return icon.replace('/big/', '/small/')},
            alternative_name: { // alternative name (for slimbox etc.)
                id: 0, // information field ID
                always: true, // always use the alternative name, not just for detail view
                tooltip: false, // also display it as the title tag on the thumbnails
                inline: false //display the alternative name in addition to the name inline, not used when "always" it true
            },
            basket: null, // object to use as a basket, initialized by a basket plugin
            state: '', // prefix for state in url (multi-instance) or false to disable state
            send: false, // add a mailto-link (ID of the download format to use or true for the first)
            extended: false, // request extended asset properties (size, modified) etc. - has no effect other than that you could use them for sorting
            stream: true, // enables audio and video streaming - use with caution, might overload the server
            autoplay: true, // whether to automatically start playing (with vimeo and/or stream)
            recursive: false, // load assets from all subnodes recursively (like keywords in 1.x)
            noderef: false, // show assets that have node referencing infofields to this node, 2.x
            infofields: [], // list of information field IDs to load with the list response (use sparingly)
            detail_trigger: true, // listen to the details=... parameter and show the assets details view (regardless of actual navigation context)
            column_class: 'anura-masonry-col-5', // class to assign to each asset, used by masonry to determine the column width (build-in are anura-masonry-col-n, n = [2..9])
            masonry: {columnWidth: '.anura-asset-container', percentPosition: true, horizontalOrder: true},
            highres_thumbs: false, // masonry upscales -> thumbs may be blurry - set this to true to use previews (up to 1024px) instead
            lightbox_rel: 'lightbox-anura' // name of the rel attribute to use for lighbox triggers
        };
    function Plugin( element, options ) {
        this.element = this.container = $(element);
        this.masonry = null;
        if (!options.lightbox_rel && options.state) options.lightbox_rel = 'lightbox-anura-'+options.state;
        this.options = $.extend( {}, defaults, options );
        this.strings = {};
        this.strings_locale = this.options.locale;
        this._defaults = defaults;
        this._name = pluginName;
        this.download_formats = null;
        this.pageCall = {};
        this.init();
    }
    Plugin.prototype = {
        init: function() {
            var _this = this;
            if (!$.isEmptyObject(this.strings) && this.options.locale === this.strings_locale) {
                this.loadGallery();
            } else {
                $.anura.getJSON(this, this.options.server+'/asset'+_this.options.ext+'callback=?&string=anura.addToBasket' +
                    '&string=anura.removeFromBasket&string=anura.loadMore&string=MENUITEMID_MAILTO',
                    {locale: this.options.locale}, 'anuraMasonryStrings', function (data) {
                        _this.strings = data;
                        _this.strings_locale = _this.options.locale;
                        _this.loadGallery();
                    });
            }
        },
        loadGallery: function() {
            var existing = this.element.find('.anura-masonry-container'), _this = this;
            if (existing.length > 0) this.container = existing.first();
            else this.container = $('<div />').addClass('anura-masonry-container').appendTo(this.element);

            this.container.empty().addClass('anura-loading')
                .removeClass(function (index, css) {return (css.match (/anura-locale-\S+/g) || []).join(' ');}).addClass('anura-locale-'+this.options.locale);
            this.element.trigger('anura-loading', [this.options]);

            if (this.options.download_directly && this.download_formats == null) this.requestDownloadFormats();

            this.pageCall = {url: this.options.server+'/node'+this.options.ext+'callback=?', name: 'anuraMasonry',
                params: {locale: this.options.locale, alt_name: this.options.alternative_name.id, extended: this.options.extended}};
            if (this.options.sort) {
                if (typeof this.options.sort === "string") {
                    var s = this.options.sort.split('_'); this.options.sort = {};
                    this.options.sort.seq = s.pop();
                    this.options.sort.order = s.join('_'); // join again for info_123
                }
                $.extend(this.pageCall.params, {sort_order: this.options.sort.order, sort_order_seq: this.options.sort.seq || 'asc'});
            }
            if (this.options.limit > 0) this.pageCall.params.limit = this.options.limit;
            if (this.options.infofields.length > 0) $.extend(this.pageCall.params, {infofields: this.options.infofields.join(',')});

            if (this.options.assets) {
                this.populateContainer(this.options.assets);
                this.element.trigger('anura-loaded', [this.options, this.options.assets]);
                return;
            } else if (this.options.search && (this.options.search.custom || this.options.search.string)) {
                this.pageCall.name = 'anuraMasonrySearch';
                this.pageCall.url = this.options.server+'/node'+this.options.ext+'callback=?'+((this.options.search.custom)?('&'+this.options.search.custom):'');
                $.extend(this.pageCall.params, {search: this.options.search.string, search_node: this.options.search.node});
                this.options.search.string = '';
            } else if (this.options.node && this.options.node.id) {
                this.pageCall.name = 'anuraMasonryNode';
                $.extend(this.pageCall.params, {kind: this.options.node.kind, id: this.options.node.id,
                    recursive: this.options.recursive, noderef: this.options.noderef});
            } else if (this.options.top && this.options.top.number) {
                this.pageCall.name = 'anuraMasonryTopDl';
                $.extend(this.pageCall.params, {top: this.options.top.number, days: this.options.top.days});
            } else if (this.options.relations && this.options.relations.id) {
                this.pageCall.name = 'anuraMasonryRel'+this.options.relations.id;
                $.extend(this.pageCall.params, {relations: this.options.relations.id});
                if (this.options.relations.version) $.extend(this.pageCall.params, {version: this.options.relations.version});
                if (this.options.relations.type) $.extend(this.pageCall.params, {type: this.options.relations.type});
            } else {
                this.container.removeClass('anura-loading');
                return;
            }
            this.element.data('page-call', this.pageCall);

            this.showPage(0);

            if (this.options.detail_trigger) { // TODO have utils do this
                var detailAssetId = $.anura.getState(this.options.state, 'details');
                if (detailAssetId) {
                    var lightboxSelector = ' a[rel*='+this.options.lightbox_rel+']'+(_this.options.detail_view === 'swipebox'?' img':'');
                    this.element.on('anura-loaded-first.detail-trigger', function() {
                        _this.element.off('anura-loaded-first.detail-trigger');
                        _this.options.detail_trigger = false; // only on first run
                        if (_this.element.find('#anura-asset-id'+detailAssetId).length === 0) {
                            $.anura.getJSON(_this, _this.pageCall.url, {search_id: detailAssetId}, 'anuraAsset'+detailAssetId, function (data, response) {
                                if (data[0]) {
                                    _this.populateContainer(data);
                                    _this.element.find('#anura-asset-id'+detailAssetId).hide().find(lightboxSelector).click();
                                }
                            });
                        } else if (_this.options.detail_view === 'anura') {
                            _this.element.anuraLightbox({action: detailAssetId});
                        } else {
                            _this.element.find('#anura-asset-id'+detailAssetId+lightboxSelector).click();
                        }
                    });
                }
            }
        },
        requestDownloadFormats: function(callback) {
            var _this = this;
            var onComplete = function(data) {
                _this.download_formats = data;
                if (_this.options.send === true) _this.options.send = data[0].id;
                if (callback) callback();
            };
            if (_this.download_formats) {
                onComplete(_this.download_formats);
            } else {
                $.anura.getJSON(this, this.options.server+'/asset'+this.options.ext+'callback=?',
                    {downloadformats: 1, locale: this.options.locale}, 'anuraMasonryDlf', onComplete);
            }
        },
        showPage: function(page) {
            var _this = this;
            this.container.addClass('anura-loading');
            if (this.options.paginate === 'auto' || _this.options.paginate === 'button') {
                this.container.find('.anura-autoload').remove();
                var autoload = $('<div/>').addClass('anura-autoload').appendTo(this.container), autoload_page = page;
                var doInfiniScroll = function() {
                    var button = autoload.find('button');
                    if (button.prop('disabled') === true) return;
                    button.prop('disabled', true);
                    autoload_page = autoload_page + 1;
                    $.extend(_this.pageCall.params, {paginate: _this.options.autoload_fetch, page: autoload_page});
                    $.anura.getJSON(_this, _this.pageCall.url, _this.pageCall.params, _this.pageCall.name, function (data, response) {
                        if (autoload_page === 1) _this.container.find('.anura-asset-container').remove();
                        var total = response && response.total ? response.total : data.length;
                        if (!response && total > _this.options.autoload_fetch) data = data.slice((autoload_page-1)*_this.options.autoload_fetch, autoload_page*_this.options.autoload_fetch); // v1 server
                        _this.populateContainer(data, _this.options.autoload_fetch);
                        button.prop('disabled', false);
                        if (_this.container.find('.anura-asset-container:visible').length >= total) autoload.remove();
                        if (autoload_page === 1) _this.element.trigger('anura-loaded-first', [_this.options, total]);
                        _this.element.trigger('anura-loaded', [_this.options, data]);
                        if (autoload.is(':appeared')) doInfiniScroll(); // do it again if it's still visible
                    }, function () {
                        autoload.remove();
                    });
                };
                if (this.options.paginate === 'auto') autoload.appear().on('appear', doInfiniScroll); // TODO we should somehow attach to the "last" asset-container, as it may be considerable higher up than the paginator
                $('<button/>').text(_this.strings['anura.loadMore'] || 'load more').addClass(_this.options.button_class).appendTo(autoload).on('click', doInfiniScroll);
                doInfiniScroll();
            } else {
                $.extend(this.pageCall.params, {paginate: this.options.paginate, page: page+1});
                $.anura.getJSON(this, this.pageCall.url, this.pageCall.params, this.pageCall.name, function(data, response) {
                    _this.container.find('.anura-asset-container').remove();
                    var total = response && response.total ? response.total : data.length;
                    if (!response && total > _this.options.paginate) data = data.slice(page*_this.options.paginate, (page+1)*_this.options.paginate); // v1 server
                    _this.populateContainer(data, _this.options.paginate);
                    $.anura.addPaginator(_this, page, total, _this.container.find('.anura-masonry-wrapper'));
                    if (page === 0) _this.element.trigger('anura-loaded-first', [_this.options, total]);
                    _this.element.trigger('anura-loaded', [_this.options, data]);
                });
            }
        },
        populateContainer: function(assets, limit) {
            var _this = this;
            if (limit && $.isNumeric(limit) && assets.length !== limit) assets = assets.slice(0, limit);
            var wrapper = this.container.find('.anura-masonry-wrapper');
            if (wrapper.length === 0) wrapper = $('<div />').addClass('anura-masonry-wrapper').prependTo(this.container);

            this.masonry = wrapper.masonry($.extend({}, this.options.masonry, {itemSelector: '.anura-asset-container'}));

            $.each(assets, function (i, item) {
                _this.appendAsset(item, wrapper);
            });
            //this.container.find('.anura-autoload').addClass('anura-asset-container anura-asset-type-0 '+this.options.column_class).appendTo(wrapper);
            //this.masonry.masonry('layout');

            this.addLightbox();
            this.container.removeClass('anura-loading');
        },
        addLightbox: function() { // TODO have utils do this
            var _this = this;
            switch(this.options.detail_view) {
                case 'slimbox':
                    this.container.find(".anura-asset-container a[rel^='lightbox']").slimbox(this.options.detail_settings, null, function(el) {
                        return (this === el) || ((this.rel.length > 8) && (this.rel === el.rel));
                    });
                    break;
                case 'swipebox':
                    if (baguetteBox !== undefined) baguetteBox.run('.anura-masonry-container', this.options.detail_settings);
                    else this.container.find(".anura-asset-container a[rel^='lightbox']").swipebox(this.options.detail_settings);
                    break;
                case 'spotlight':
                    $.each(_this.options.detail_settings, function(key, value) {_this.container.attr('data-'+key, value)});
                    _this.container.addClass('spotlight-group'); // setting .data(_this.options.detail_settings) doesn't work?
                    _this.container.find(".anura-asset-container a[rel^='lightbox']").addClass('spotlight');
                    break;
                case 'anura':
                    _this.element.anuraLightbox($.extend({}, _this.options.detail_settings, {
                        server: _this.options.server,
                        locale: _this.options.locale,
                        basket: _this.options.basket,
                        trigger: 'a[rel^='+_this.options.lightbox_rel+']',
                        state: _this.options.state,
                        pageCall: _this.pageCall
                    }));
                    break;
                case 'details':
                    $('#colorbox').addClass('hasAnuraDetails');
                case 'colorbox':
                    var infopanel = function(a) {
                        if (_this.options.detail_view !== 'details') return;
                        var panel = $('#anuraInfoPanel');
                        if (!panel.length) {
                            panel = $('<div id="anuraInfoPanel"/>').appendTo('#cboxContent');
                            $('<button type="button" id="cboxInfo"/>').text('info').on('click', function() {
                                $('#anuraInfoPanel').toggle();
                            }).insertBefore($('#cboxClose'));
                        }
                        if (_this.options.colorbox_min_height) var minHeight = Math.min($(window).height() - 50, _this.options.colorbox_min_height); // workaround for missing minHeight option
                        if ($('#cboxLoadedContent').height() < minHeight) $.colorbox.resize({innerHeight: minHeight+'px'});
                        panel.height($('#cboxLoadedContent').height());
                        if ($(window).width() > 767) panel.show(); //argh
                        var asset_container = $.colorbox.element().closest('.anura-asset-container');
                        var options = $.extend({send: _this.options.send, download_directly: _this.options.download_directly}, _this.options.detail_settings,
                            {asset: asset_container.data('assetId') || asset_container.attr('id').replace(/\D/g,''), alternative_name: _this.options.alternative_name.id,
                             server: _this.options.server,locale: _this.options.locale, basket: _this.options.basket});
                        panel.anuraDetails(options);
                    };
                    var colorbox_options = $.extend({transition: 'none', current: '{current} / {total}'}, _this.options.colorbox_settings,
                        {rel: this.options.lightbox_rel, onComplete: infopanel,
                            onCleanup: function() {$('#anuraInfoPanel').fadeOut()},
                            onClosed: function() {$.anura.removeState(_this.options.state, 'details');}
                        });
                    this.container.find('.anura-asset-container a.image').colorbox($.extend(colorbox_options, {maxWidth: '98%', maxHeight: '98%'}));
                    this.container.find('.anura-asset-container a.vimeo, .anura-asset-container a.youtube').colorbox($.extend(colorbox_options, {iframe:true, innerWidth:960, innerHeight:540}));
                    this.container.find('.anura-asset-container a.soundcloud').colorbox($.extend(colorbox_options, {iframe:true, innerWidth:500, innerHeight:500}));
                    this.container.find('.anura-asset-container a.player').colorbox($.extend(colorbox_options, {iframe:true, innerWidth:640, innerHeight:480}));
                    break;
            }
        },
        appendAsset: function(item, target) {
            var _this = this;
            if (item.type > 100) {
                var container = $('<div />').data('asset-id', item.id)
                    .addClass('anura-asset-container anura-asset-type-'+item.type+' '+this.options.column_class).appendTo(target);
                var holder = $('<div />').addClass('anura-node-container').appendTo(container)
                    .on('click', function() {
                        if ($('#anura-node-'+item.id).length) $('#anura-node-'+item.id+' > a').click();
                        _this.element.anuraMasonry({node: {id: item.id, kind: item.type}});
                    });
                var tag = (item.custom?item.custom:'!default')+'/large.png';
                var tag_type = (this.options.node.kind === 103?'keyword':(this.options.node.kind === 102?'assetcollection':'folder'));
                $('<img>').attr('src', this.options.server+'/../img/aci/'+tag_type+'/'+tag).addClass('anura-node').appendTo(holder);
                $('<p />').addClass('anura-node-name').text(item.name).attr('title', item.name).appendTo(container);
                return;
            }
            var name = this.options.alternative_name.always?item.alt_name:item.name,
                tooltip = this.options.alternative_name.tooltip?item.alt_name:item.name;
            var container = $('<div />').data('asset-id', item.id).attr('id', 'anura-asset-id'+item.id)
                .addClass('anura-asset-container anura-asset-type-'+item.type+' '+this.options.column_class).hide().appendTo(target);
            if (item.infofield) container.data('asset-infofield', item.infofield); // legacy
            // TODO move this to utils
            if (item.infofields) $.each(item.infofields, function(infoId, infoVal) {container.data('asset-infofield-'+infoId, infoVal);});
            if (item.infofields_raw) $.each(item.infofields_raw, function(infoId, infoVal) {container.data('asset-infofield-raw-'+infoId, infoVal);});
            if (item.downloadable === false) container.addClass('anura-not-downloadable');
            if (item.version) container.data('asset-version', item.version).data('asset-versioned', item.versioned);
            if (item.extension) container.data('asset-extension', item.extension);
            if (item.parents) container.data('asset-parents', item.parents);
            if (item.custom) $.each(item.custom.replace(/_/g, '-').split(' '), function(index, value) {container.addClass('anura-custom-'+value);});
            if (_this.options.extended) container.data('asset-extended', item);
            if (this.options.download_menu) this.addMenu(container, item);
            if (this.options.vimeo_legacy && (item.infofield || item.vimeo)) {
				if (!item.streaming) item.streaming = {};
				if (!item.streaming.vimeo) item.streaming.vimeo = item.infofield || item.vimeo;
			}
            var holder = $('<div />').addClass('anura-image-container').appendTo(container);
            var img = $('<img />').data('id', item.id).attr('alt', tooltip).addClass('anura-image') /* IE */.removeAttr('width').removeAttr('height')/* /IE */;
            var imgSrc = this.options.server+'/asset'+this.options.ext + 'thumb=' + item.id + (_this.options.highres_thumbs?'&big=true':'') + ($.anura.tokenProvider?'&token='+$.anura.tokenProvider():'') + '&img=a.jpg';
            img.imagesLoaded(function(e) {
                $(this.elements).fadeIn(150);
                _this.masonry.masonry('appended', container.show());
                _this.masonry.masonry('layout');
            }).attr('src', imgSrc);
            if (this.options.detail_view !== 'none') {
                var trigger = $('<a />').attr('rel', this.options.lightbox_rel+' nofollow').data('asset-id', item.id).data('asset-extension', item.extension)
                    .attr('title', tooltip).attr('target', '_blank').appendTo(holder);
                if (this.options.detail_view === 'download') {
                    trigger.attr('href', this.options.server+'/asset'+this.options.ext+'download='+item.id+'&format='+this.options.detail_settings.format);
                } else if (item.streaming && item.streaming.soundcloud && item.type === 4) {
                    trigger.addClass('soundcloud').attr('href', window.location.protocol+'//w.soundcloud.com/player/?url=http://api.soundcloud.com/tracks/'+item.streaming.soundcloud+'?secret_token='+item.streaming.soundcloud_key+'&visual=true'+(this.options.autoplay?'&auto_play=true':'')+'&buying=false&download=false&show_artwork=true&show_comments=false&show_playcount=false&show_user=false&hide_related=true');
                } else if (item.streaming && item.streaming.vimeo && item.type === 3) {
                    trigger.addClass('vimeo').attr('href', window.location.protocol+'//player.vimeo.com/video/'+item.streaming.vimeo+(this.options.autoplay?'?autoplay=1':''));
                } else if (item.streaming && item.streaming.youtube && item.type === 3) {
                    trigger.addClass('youtube').attr('href', window.location.protocol+'//www.youtube.com/embed/'+item.streaming.youtube+(this.options.autoplay?'?autoplay=1':''));
                } else if (this.options.stream && (item.type === 3 || item.type === 4)) {
                    trigger.addClass('player').attr('href', this.options.server+'/asset'+this.options.ext+'play='+item.id+(this.options.autoplay?'&autoplay=true':'')+($.anura.tokenProvider?'&token='+$.anura.tokenProvider():''));
                } else {
                    trigger.addClass('image').attr('href', this.options.server+'/asset'+this.options.ext+'preview='+item.id+($.anura.tokenProvider?'&token='+$.anura.tokenProvider():'')+'&img=a.jpg');
                }
                if ($.isFunction(this.options.detail_view)) trigger.on('click', function(e) {e.preventDefault(); _this.options.detail_view(item, trigger)});
                img.appendTo(trigger);
            } else {
                img.appendTo(holder);
            }
            $('<p />').addClass('anura-asset-name').text(name).attr('title', name).appendTo(container);
            if (!this.options.alternative_name.always && this.options.alternative_name.inline && item.alt_name !== item.name)
                $('<p />').addClass('anura-asset-alternative-name').text(item.alt_name).attr('title', item.alt_name).appendTo(container);
            return container;
        },
        addMenu: function(target, asset) {
            var _this = this;
            var menu = $('<div />').addClass('anura-asset-menu').appendTo(target);
            var holder = $('<div />').addClass('anura-asset-menu-holder').appendTo(menu);
            if (this.options.download_directly && this.download_formats == null) {
                this.requestDownloadFormats(function() {
                    _this.populateMenu(holder, asset);
                });
            } else {
                this.populateMenu(holder, asset);
            }
            target.on('touchstart', function() {
                _this.container.find('.anura-asset-container.anura-touch-hover')
                    .not('.asset'+$(this).data('asset-id')).removeClass('anura-touch-hover');
                $(this).toggleClass('anura-touch-hover');
            });
        },
        populateMenu: function(holder, asset) {
            var _this = this;
            if (this.options.basket) {
                var b_link = $('<a />').addClass('anura-add-basket').attr('title', _this.strings['anura.addToBasket'] || 'add to basket')
                    .attr('id', 'basket'+asset.id).appendTo(holder)
                    .on('click', function() {
                        if (b_link.hasClass('added')) {
                            _this.options.basket.anuraBasket({remove: asset.id});
                            b_link.attr('title', _this.strings['anura.addToBasket'] || 'add to basket');
                            b_link.removeClass('added');
                        } else {
                            _this.options.basket.anuraBasket({add: asset.id, force_zip: _this.options.force_zip});
                            b_link.attr('title', _this.strings['anura.removeFromBasket'] || 'remove from basket');
                            b_link.addClass('added');
                        }
                    });
                $('<span />').addClass('anura-asset-menu-button anura-asset-menu-basket').appendTo(b_link);
                if ($.inArray(asset.id, this.options.basket.data('assets')) !== -1) b_link.addClass('added');
            }
            if (this.options.send) {
                var send = $('<a />').attr('rel', 'nofollow').appendTo(holder)
                    .attr('href', 'mailto:?body='+encodeURIComponent(_this.options.server+'/asset'+_this.options.ext
                        +'download='+asset.id+'&format='+this.options.send+'&locale='+_this.options.locale
                        +(_this.options.force_zip?'&forcezip=true':'') + ($.anura.tokenProvider ? '&token=' + $.anura.tokenProvider() : '')));
                $('<span />').addClass('anura-asset-menu-button anura-asset-menu-mailto')
                    .attr('title', _this.strings['MENUITEMID_MAILTO'] || 'send hyperlink').appendTo(send);
            }
            if (this.options.download_directly) {
                $.each(this.download_formats, function(index, format) {
                    if (format.extensions && asset.extension && $.inArray(asset.extension, format.extensions) === -1) return;
                    var base = (format.url)?(format.url+'?'):('asset'+_this.options.ext+'format='+format.id+'&');
                    var link = $('<a />').attr('title', format.name).attr('target', '_blank').attr('rel', 'nofollow').appendTo(holder);
                    if (format.features && $.inArray('editable', format.features) !== -1) {
                        _this.element.anuraCropper({server: _this.options.server, locale: _this.options.locale, assetId: asset.id, downloadFormat: format.id, mode: 'download'});
                    } else {
                        link.attr('href', _this.options.server+'/'+base+'download='+asset.id+'&locale='+_this.options.locale
                            +(_this.options.force_zip?'&forcezip=true':'') + ($.anura.tokenProvider ? '&token=' + $.anura.tokenProvider() : ''))
                            .on('click', function () {$(window).trigger('anura-download', [asset.id, format.id]);});
                    }
                    $('<img />').addClass('anura-asset-menu-button').attr('src', _this.options.server+'/'+_this.options.icon_transform(format.icon)).appendTo(link);
                });
            }
        }
    };
    $.fn[pluginName] = function ( options ) {
        return this.each(function (i, item) { // iterate matches
            var instance = $.data(item, "plugin_" + pluginName);
            if (!instance) { // if it has no instance yet, create it
                $.data(item, "plugin_" + pluginName, new Plugin( item, options ));
            } else { // otherwise merge new options with existing ones, empty and re-init
                instance.options = $.extend( {}, instance.options, options );
                instance.init();
            }
        });
    };
})( jQuery, window, document );

/*!
 * jQuery appear plugin
 * Copyright (c) 2012 Andrey Sidorov, licensed under MIT license.
 * https://github.com/morr/jquery.appear, version: 0.3.6
 */
(function($){var selectors=[];var check_binded=false;var check_lock=false;var defaults={interval:250,force_process:false};var $window=$(window);var $prior_appeared=[];function process(){check_lock=false;for(var index=0,selectorsLength=selectors.length;index<selectorsLength;index++){var $appeared=$(selectors[index]).filter(function(){return $(this).is(':appeared')});$appeared.trigger('appear',[$appeared]);if($prior_appeared[index]){var $disappeared=$prior_appeared[index].not($appeared);$disappeared.trigger('disappear',[$disappeared])}$prior_appeared[index]=$appeared}};function add_selector(selector){selectors.push(selector);$prior_appeared.push()}$.expr[':']['appeared']=function(element){var $element=$(element);if(!$element.is(':visible')){return false}var window_left=$window.scrollLeft();var window_top=$window.scrollTop();var offset=$element.offset();var left=offset.left;var top=offset.top;if(top+$element.height()>=window_top&&top-($element.data('appear-top-offset')||0)<=window_top+$window.height()&&left+$element.width()>=window_left&&left-($element.data('appear-left-offset')||0)<=window_left+$window.width()){return true}else{return false}};$.fn.extend({appear:function(options){var opts=$.extend({},defaults,options||{});var selector=this.selector||this;if(!check_binded){var on_check=function(){if(check_lock){return}check_lock=true;setTimeout(process,opts.interval)};$(window).scroll(on_check).resize(on_check);check_binded=true}if(opts.force_process){setTimeout(process,opts.interval)}add_selector(selector);return $(selector)}});$.extend({force_appear:function(){if(check_binded){process();return true}return false}})})(function(){if(typeof module!=='undefined'){return require('jquery')}else{return jQuery}}());

/*!
 * imagesLoaded PACKAGED v4.1.4
 * JavaScript is all like "You images are done yet or what?"
 * MIT License
 */
!function(e,t){"function"==typeof define&&define.amd?define("ev-emitter/ev-emitter",t):"object"==typeof module&&module.exports?module.exports=t():e.EvEmitter=t()}("undefined"!=typeof window?window:this,function(){function e(){}var t=e.prototype;return t.on=function(e,t){if(e&&t){var i=this._events=this._events||{},n=i[e]=i[e]||[];return n.indexOf(t)==-1&&n.push(t),this}},t.once=function(e,t){if(e&&t){this.on(e,t);var i=this._onceEvents=this._onceEvents||{},n=i[e]=i[e]||{};return n[t]=!0,this}},t.off=function(e,t){var i=this._events&&this._events[e];if(i&&i.length){var n=i.indexOf(t);return n!=-1&&i.splice(n,1),this}},t.emitEvent=function(e,t){var i=this._events&&this._events[e];if(i&&i.length){i=i.slice(0),t=t||[];for(var n=this._onceEvents&&this._onceEvents[e],o=0;o<i.length;o++){var r=i[o],s=n&&n[r];s&&(this.off(e,r),delete n[r]),r.apply(this,t)}return this}},t.allOff=function(){delete this._events,delete this._onceEvents},e}),function(e,t){"use strict";"function"==typeof define&&define.amd?define(["ev-emitter/ev-emitter"],function(i){return t(e,i)}):"object"==typeof module&&module.exports?module.exports=t(e,require("ev-emitter")):e.imagesLoaded=t(e,e.EvEmitter)}("undefined"!=typeof window?window:this,function(e,t){function i(e,t){for(var i in t)e[i]=t[i];return e}function n(e){if(Array.isArray(e))return e;var t="object"==typeof e&&"number"==typeof e.length;return t?d.call(e):[e]}function o(e,t,r){if(!(this instanceof o))return new o(e,t,r);var s=e;return"string"==typeof e&&(s=document.querySelectorAll(e)),s?(this.elements=n(s),this.options=i({},this.options),"function"==typeof t?r=t:i(this.options,t),r&&this.on("always",r),this.getImages(),h&&(this.jqDeferred=new h.Deferred),void setTimeout(this.check.bind(this))):void a.error("Bad element for imagesLoaded "+(s||e))}function r(e){this.img=e}function s(e,t){this.url=e,this.element=t,this.img=new Image}var h=e.jQuery,a=e.console,d=Array.prototype.slice;o.prototype=Object.create(t.prototype),o.prototype.options={},o.prototype.getImages=function(){this.images=[],this.elements.forEach(this.addElementImages,this)},o.prototype.addElementImages=function(e){"IMG"==e.nodeName&&this.addImage(e),this.options.background===!0&&this.addElementBackgroundImages(e);var t=e.nodeType;if(t&&u[t]){for(var i=e.querySelectorAll("img"),n=0;n<i.length;n++){var o=i[n];this.addImage(o)}if("string"==typeof this.options.background){var r=e.querySelectorAll(this.options.background);for(n=0;n<r.length;n++){var s=r[n];this.addElementBackgroundImages(s)}}}};var u={1:!0,9:!0,11:!0};return o.prototype.addElementBackgroundImages=function(e){var t=getComputedStyle(e);if(t)for(var i=/url\((['"])?(.*?)\1\)/gi,n=i.exec(t.backgroundImage);null!==n;){var o=n&&n[2];o&&this.addBackground(o,e),n=i.exec(t.backgroundImage)}},o.prototype.addImage=function(e){var t=new r(e);this.images.push(t)},o.prototype.addBackground=function(e,t){var i=new s(e,t);this.images.push(i)},o.prototype.check=function(){function e(e,i,n){setTimeout(function(){t.progress(e,i,n)})}var t=this;return this.progressedCount=0,this.hasAnyBroken=!1,this.images.length?void this.images.forEach(function(t){t.once("progress",e),t.check()}):void this.complete()},o.prototype.progress=function(e,t,i){this.progressedCount++,this.hasAnyBroken=this.hasAnyBroken||!e.isLoaded,this.emitEvent("progress",[this,e,t]),this.jqDeferred&&this.jqDeferred.notify&&this.jqDeferred.notify(this,e),this.progressedCount==this.images.length&&this.complete(),this.options.debug&&a&&a.log("progress: "+i,e,t)},o.prototype.complete=function(){var e=this.hasAnyBroken?"fail":"done";if(this.isComplete=!0,this.emitEvent(e,[this]),this.emitEvent("always",[this]),this.jqDeferred){var t=this.hasAnyBroken?"reject":"resolve";this.jqDeferred[t](this)}},r.prototype=Object.create(t.prototype),r.prototype.check=function(){var e=this.getIsImageComplete();return e?void this.confirm(0!==this.img.naturalWidth,"naturalWidth"):(this.proxyImage=new Image,this.proxyImage.addEventListener("load",this),this.proxyImage.addEventListener("error",this),this.img.addEventListener("load",this),this.img.addEventListener("error",this),void(this.proxyImage.src=this.img.src))},r.prototype.getIsImageComplete=function(){return this.img.complete&&this.img.naturalWidth},r.prototype.confirm=function(e,t){this.isLoaded=e,this.emitEvent("progress",[this,this.img,t])},r.prototype.handleEvent=function(e){var t="on"+e.type;this[t]&&this[t](e)},r.prototype.onload=function(){this.confirm(!0,"onload"),this.unbindEvents()},r.prototype.onerror=function(){this.confirm(!1,"onerror"),this.unbindEvents()},r.prototype.unbindEvents=function(){this.proxyImage.removeEventListener("load",this),this.proxyImage.removeEventListener("error",this),this.img.removeEventListener("load",this),this.img.removeEventListener("error",this)},s.prototype=Object.create(r.prototype),s.prototype.check=function(){this.img.addEventListener("load",this),this.img.addEventListener("error",this),this.img.src=this.url;var e=this.getIsImageComplete();e&&(this.confirm(0!==this.img.naturalWidth,"naturalWidth"),this.unbindEvents())},s.prototype.unbindEvents=function(){this.img.removeEventListener("load",this),this.img.removeEventListener("error",this)},s.prototype.confirm=function(e,t){this.isLoaded=e,this.emitEvent("progress",[this,this.element,t])},o.makeJQueryPlugin=function(t){t=t||e.jQuery,t&&(h=t,h.fn.imagesLoaded=function(e,t){var i=new o(this,e,t);return i.jqDeferred.promise(h(this))})},o.makeJQueryPlugin(),o});

/*!
 * Masonry PACKAGED v4.2.2
 * Cascading grid layout library
 * https://masonry.desandro.com
 * MIT License
 * by David DeSandro
 */
!function(t,e){"function"==typeof define&&define.amd?define("jquery-bridget/jquery-bridget",["jquery"],function(i){return e(t,i)}):"object"==typeof module&&module.exports?module.exports=e(t,require("jquery")):t.jQueryBridget=e(t,t.jQuery)}(window,function(t,e){"use strict";function i(i,r,a){function h(t,e,n){var o,r="$()."+i+'("'+e+'")';return t.each(function(t,h){var u=a.data(h,i);if(!u)return void s(i+" not initialized. Cannot call methods, i.e. "+r);var d=u[e];if(!d||"_"==e.charAt(0))return void s(r+" is not a valid method");var l=d.apply(u,n);o=void 0===o?l:o}),void 0!==o?o:t}function u(t,e){t.each(function(t,n){var o=a.data(n,i);o?(o.option(e),o._init()):(o=new r(n,e),a.data(n,i,o))})}a=a||e||t.jQuery,a&&(r.prototype.option||(r.prototype.option=function(t){a.isPlainObject(t)&&(this.options=a.extend(!0,this.options,t))}),a.fn[i]=function(t){if("string"==typeof t){var e=o.call(arguments,1);return h(this,t,e)}return u(this,t),this},n(a))}function n(t){!t||t&&t.bridget||(t.bridget=i)}var o=Array.prototype.slice,r=t.console,s="undefined"==typeof r?function(){}:function(t){r.error(t)};return n(e||t.jQuery),i}),function(t,e){"function"==typeof define&&define.amd?define("ev-emitter/ev-emitter",e):"object"==typeof module&&module.exports?module.exports=e():t.EvEmitter=e()}("undefined"!=typeof window?window:this,function(){function t(){}var e=t.prototype;return e.on=function(t,e){if(t&&e){var i=this._events=this._events||{},n=i[t]=i[t]||[];return-1==n.indexOf(e)&&n.push(e),this}},e.once=function(t,e){if(t&&e){this.on(t,e);var i=this._onceEvents=this._onceEvents||{},n=i[t]=i[t]||{};return n[e]=!0,this}},e.off=function(t,e){var i=this._events&&this._events[t];if(i&&i.length){var n=i.indexOf(e);return-1!=n&&i.splice(n,1),this}},e.emitEvent=function(t,e){var i=this._events&&this._events[t];if(i&&i.length){i=i.slice(0),e=e||[];for(var n=this._onceEvents&&this._onceEvents[t],o=0;o<i.length;o++){var r=i[o],s=n&&n[r];s&&(this.off(t,r),delete n[r]),r.apply(this,e)}return this}},e.allOff=function(){delete this._events,delete this._onceEvents},t}),function(t,e){"function"==typeof define&&define.amd?define("get-size/get-size",e):"object"==typeof module&&module.exports?module.exports=e():t.getSize=e()}(window,function(){"use strict";function t(t){var e=parseFloat(t),i=-1==t.indexOf("%")&&!isNaN(e);return i&&e}function e(){}function i(){for(var t={width:0,height:0,innerWidth:0,innerHeight:0,outerWidth:0,outerHeight:0},e=0;u>e;e++){var i=h[e];t[i]=0}return t}function n(t){var e=getComputedStyle(t);return e||a("Style returned "+e+". Are you running this code in a hidden iframe on Firefox? See https://bit.ly/getsizebug1"),e}function o(){if(!d){d=!0;var e=document.createElement("div");e.style.width="200px",e.style.padding="1px 2px 3px 4px",e.style.borderStyle="solid",e.style.borderWidth="1px 2px 3px 4px",e.style.boxSizing="border-box";var i=document.body||document.documentElement;i.appendChild(e);var o=n(e);s=200==Math.round(t(o.width)),r.isBoxSizeOuter=s,i.removeChild(e)}}function r(e){if(o(),"string"==typeof e&&(e=document.querySelector(e)),e&&"object"==typeof e&&e.nodeType){var r=n(e);if("none"==r.display)return i();var a={};a.width=e.offsetWidth,a.height=e.offsetHeight;for(var d=a.isBorderBox="border-box"==r.boxSizing,l=0;u>l;l++){var c=h[l],f=r[c],m=parseFloat(f);a[c]=isNaN(m)?0:m}var p=a.paddingLeft+a.paddingRight,g=a.paddingTop+a.paddingBottom,y=a.marginLeft+a.marginRight,v=a.marginTop+a.marginBottom,_=a.borderLeftWidth+a.borderRightWidth,z=a.borderTopWidth+a.borderBottomWidth,E=d&&s,b=t(r.width);b!==!1&&(a.width=b+(E?0:p+_));var x=t(r.height);return x!==!1&&(a.height=x+(E?0:g+z)),a.innerWidth=a.width-(p+_),a.innerHeight=a.height-(g+z),a.outerWidth=a.width+y,a.outerHeight=a.height+v,a}}var s,a="undefined"==typeof console?e:function(t){console.error(t)},h=["paddingLeft","paddingRight","paddingTop","paddingBottom","marginLeft","marginRight","marginTop","marginBottom","borderLeftWidth","borderRightWidth","borderTopWidth","borderBottomWidth"],u=h.length,d=!1;return r}),function(t,e){"use strict";"function"==typeof define&&define.amd?define("desandro-matches-selector/matches-selector",e):"object"==typeof module&&module.exports?module.exports=e():t.matchesSelector=e()}(window,function(){"use strict";var t=function(){var t=window.Element.prototype;if(t.matches)return"matches";if(t.matchesSelector)return"matchesSelector";for(var e=["webkit","moz","ms","o"],i=0;i<e.length;i++){var n=e[i],o=n+"MatchesSelector";if(t[o])return o}}();return function(e,i){return e[t](i)}}),function(t,e){"function"==typeof define&&define.amd?define("fizzy-ui-utils/utils",["desandro-matches-selector/matches-selector"],function(i){return e(t,i)}):"object"==typeof module&&module.exports?module.exports=e(t,require("desandro-matches-selector")):t.fizzyUIUtils=e(t,t.matchesSelector)}(window,function(t,e){var i={};i.extend=function(t,e){for(var i in e)t[i]=e[i];return t},i.modulo=function(t,e){return(t%e+e)%e};var n=Array.prototype.slice;i.makeArray=function(t){if(Array.isArray(t))return t;if(null===t||void 0===t)return[];var e="object"==typeof t&&"number"==typeof t.length;return e?n.call(t):[t]},i.removeFrom=function(t,e){var i=t.indexOf(e);-1!=i&&t.splice(i,1)},i.getParent=function(t,i){for(;t.parentNode&&t!=document.body;)if(t=t.parentNode,e(t,i))return t},i.getQueryElement=function(t){return"string"==typeof t?document.querySelector(t):t},i.handleEvent=function(t){var e="on"+t.type;this[e]&&this[e](t)},i.filterFindElements=function(t,n){t=i.makeArray(t);var o=[];return t.forEach(function(t){if(t instanceof HTMLElement){if(!n)return void o.push(t);e(t,n)&&o.push(t);for(var i=t.querySelectorAll(n),r=0;r<i.length;r++)o.push(i[r])}}),o},i.debounceMethod=function(t,e,i){i=i||100;var n=t.prototype[e],o=e+"Timeout";t.prototype[e]=function(){var t=this[o];clearTimeout(t);var e=arguments,r=this;this[o]=setTimeout(function(){n.apply(r,e),delete r[o]},i)}},i.docReady=function(t){var e=document.readyState;"complete"==e||"interactive"==e?setTimeout(t):document.addEventListener("DOMContentLoaded",t)},i.toDashed=function(t){return t.replace(/(.)([A-Z])/g,function(t,e,i){return e+"-"+i}).toLowerCase()};var o=t.console;return i.htmlInit=function(e,n){i.docReady(function(){var r=i.toDashed(n),s="data-"+r,a=document.querySelectorAll("["+s+"]"),h=document.querySelectorAll(".js-"+r),u=i.makeArray(a).concat(i.makeArray(h)),d=s+"-options",l=t.jQuery;u.forEach(function(t){var i,r=t.getAttribute(s)||t.getAttribute(d);try{i=r&&JSON.parse(r)}catch(a){return void(o&&o.error("Error parsing "+s+" on "+t.className+": "+a))}var h=new e(t,i);l&&l.data(t,n,h)})})},i}),function(t,e){"function"==typeof define&&define.amd?define("outlayer/item",["ev-emitter/ev-emitter","get-size/get-size"],e):"object"==typeof module&&module.exports?module.exports=e(require("ev-emitter"),require("get-size")):(t.Outlayer={},t.Outlayer.Item=e(t.EvEmitter,t.getSize))}(window,function(t,e){"use strict";function i(t){for(var e in t)return!1;return e=null,!0}function n(t,e){t&&(this.element=t,this.layout=e,this.position={x:0,y:0},this._create())}function o(t){return t.replace(/([A-Z])/g,function(t){return"-"+t.toLowerCase()})}var r=document.documentElement.style,s="string"==typeof r.transition?"transition":"WebkitTransition",a="string"==typeof r.transform?"transform":"WebkitTransform",h={WebkitTransition:"webkitTransitionEnd",transition:"transitionend"}[s],u={transform:a,transition:s,transitionDuration:s+"Duration",transitionProperty:s+"Property",transitionDelay:s+"Delay"},d=n.prototype=Object.create(t.prototype);d.constructor=n,d._create=function(){this._transn={ingProperties:{},clean:{},onEnd:{}},this.css({position:"absolute"})},d.handleEvent=function(t){var e="on"+t.type;this[e]&&this[e](t)},d.getSize=function(){this.size=e(this.element)},d.css=function(t){var e=this.element.style;for(var i in t){var n=u[i]||i;e[n]=t[i]}},d.getPosition=function(){var t=getComputedStyle(this.element),e=this.layout._getOption("originLeft"),i=this.layout._getOption("originTop"),n=t[e?"left":"right"],o=t[i?"top":"bottom"],r=parseFloat(n),s=parseFloat(o),a=this.layout.size;-1!=n.indexOf("%")&&(r=r/100*a.width),-1!=o.indexOf("%")&&(s=s/100*a.height),r=isNaN(r)?0:r,s=isNaN(s)?0:s,r-=e?a.paddingLeft:a.paddingRight,s-=i?a.paddingTop:a.paddingBottom,this.position.x=r,this.position.y=s},d.layoutPosition=function(){var t=this.layout.size,e={},i=this.layout._getOption("originLeft"),n=this.layout._getOption("originTop"),o=i?"paddingLeft":"paddingRight",r=i?"left":"right",s=i?"right":"left",a=this.position.x+t[o];e[r]=this.getXValue(a),e[s]="";var h=n?"paddingTop":"paddingBottom",u=n?"top":"bottom",d=n?"bottom":"top",l=this.position.y+t[h];e[u]=this.getYValue(l),e[d]="",this.css(e),this.emitEvent("layout",[this])},d.getXValue=function(t){var e=this.layout._getOption("horizontal");return this.layout.options.percentPosition&&!e?t/this.layout.size.width*100+"%":t+"px"},d.getYValue=function(t){var e=this.layout._getOption("horizontal");return this.layout.options.percentPosition&&e?t/this.layout.size.height*100+"%":t+"px"},d._transitionTo=function(t,e){this.getPosition();var i=this.position.x,n=this.position.y,o=t==this.position.x&&e==this.position.y;if(this.setPosition(t,e),o&&!this.isTransitioning)return void this.layoutPosition();var r=t-i,s=e-n,a={};a.transform=this.getTranslate(r,s),this.transition({to:a,onTransitionEnd:{transform:this.layoutPosition},isCleaning:!0})},d.getTranslate=function(t,e){var i=this.layout._getOption("originLeft"),n=this.layout._getOption("originTop");return t=i?t:-t,e=n?e:-e,"translate3d("+t+"px, "+e+"px, 0)"},d.goTo=function(t,e){this.setPosition(t,e),this.layoutPosition()},d.moveTo=d._transitionTo,d.setPosition=function(t,e){this.position.x=parseFloat(t),this.position.y=parseFloat(e)},d._nonTransition=function(t){this.css(t.to),t.isCleaning&&this._removeStyles(t.to);for(var e in t.onTransitionEnd)t.onTransitionEnd[e].call(this)},d.transition=function(t){if(!parseFloat(this.layout.options.transitionDuration))return void this._nonTransition(t);var e=this._transn;for(var i in t.onTransitionEnd)e.onEnd[i]=t.onTransitionEnd[i];for(i in t.to)e.ingProperties[i]=!0,t.isCleaning&&(e.clean[i]=!0);if(t.from){this.css(t.from);var n=this.element.offsetHeight;n=null}this.enableTransition(t.to),this.css(t.to),this.isTransitioning=!0};var l="opacity,"+o(a);d.enableTransition=function(){if(!this.isTransitioning){var t=this.layout.options.transitionDuration;t="number"==typeof t?t+"ms":t,this.css({transitionProperty:l,transitionDuration:t,transitionDelay:this.staggerDelay||0}),this.element.addEventListener(h,this,!1)}},d.onwebkitTransitionEnd=function(t){this.ontransitionend(t)},d.onotransitionend=function(t){this.ontransitionend(t)};var c={"-webkit-transform":"transform"};d.ontransitionend=function(t){if(t.target===this.element){var e=this._transn,n=c[t.propertyName]||t.propertyName;if(delete e.ingProperties[n],i(e.ingProperties)&&this.disableTransition(),n in e.clean&&(this.element.style[t.propertyName]="",delete e.clean[n]),n in e.onEnd){var o=e.onEnd[n];o.call(this),delete e.onEnd[n]}this.emitEvent("transitionEnd",[this])}},d.disableTransition=function(){this.removeTransitionStyles(),this.element.removeEventListener(h,this,!1),this.isTransitioning=!1},d._removeStyles=function(t){var e={};for(var i in t)e[i]="";this.css(e)};var f={transitionProperty:"",transitionDuration:"",transitionDelay:""};return d.removeTransitionStyles=function(){this.css(f)},d.stagger=function(t){t=isNaN(t)?0:t,this.staggerDelay=t+"ms"},d.removeElem=function(){this.element.parentNode.removeChild(this.element),this.css({display:""}),this.emitEvent("remove",[this])},d.remove=function(){return s&&parseFloat(this.layout.options.transitionDuration)?(this.once("transitionEnd",function(){this.removeElem()}),void this.hide()):void this.removeElem()},d.reveal=function(){delete this.isHidden,this.css({display:""});var t=this.layout.options,e={},i=this.getHideRevealTransitionEndProperty("visibleStyle");e[i]=this.onRevealTransitionEnd,this.transition({from:t.hiddenStyle,to:t.visibleStyle,isCleaning:!0,onTransitionEnd:e})},d.onRevealTransitionEnd=function(){this.isHidden||this.emitEvent("reveal")},d.getHideRevealTransitionEndProperty=function(t){var e=this.layout.options[t];if(e.opacity)return"opacity";for(var i in e)return i},d.hide=function(){this.isHidden=!0,this.css({display:""});var t=this.layout.options,e={},i=this.getHideRevealTransitionEndProperty("hiddenStyle");e[i]=this.onHideTransitionEnd,this.transition({from:t.visibleStyle,to:t.hiddenStyle,isCleaning:!0,onTransitionEnd:e})},d.onHideTransitionEnd=function(){this.isHidden&&(this.css({display:"none"}),this.emitEvent("hide"))},d.destroy=function(){this.css({position:"",left:"",right:"",top:"",bottom:"",transition:"",transform:""})},n}),function(t,e){"use strict";"function"==typeof define&&define.amd?define("outlayer/outlayer",["ev-emitter/ev-emitter","get-size/get-size","fizzy-ui-utils/utils","./item"],function(i,n,o,r){return e(t,i,n,o,r)}):"object"==typeof module&&module.exports?module.exports=e(t,require("ev-emitter"),require("get-size"),require("fizzy-ui-utils"),require("./item")):t.Outlayer=e(t,t.EvEmitter,t.getSize,t.fizzyUIUtils,t.Outlayer.Item)}(window,function(t,e,i,n,o){"use strict";function r(t,e){var i=n.getQueryElement(t);if(!i)return void(h&&h.error("Bad element for "+this.constructor.namespace+": "+(i||t)));this.element=i,u&&(this.$element=u(this.element)),this.options=n.extend({},this.constructor.defaults),this.option(e);var o=++l;this.element.outlayerGUID=o,c[o]=this,this._create();var r=this._getOption("initLayout");r&&this.layout()}function s(t){function e(){t.apply(this,arguments)}return e.prototype=Object.create(t.prototype),e.prototype.constructor=e,e}function a(t){if("number"==typeof t)return t;var e=t.match(/(^\d*\.?\d*)(\w*)/),i=e&&e[1],n=e&&e[2];if(!i.length)return 0;i=parseFloat(i);var o=m[n]||1;return i*o}var h=t.console,u=t.jQuery,d=function(){},l=0,c={};r.namespace="outlayer",r.Item=o,r.defaults={containerStyle:{position:"relative"},initLayout:!0,originLeft:!0,originTop:!0,resize:!0,resizeContainer:!0,transitionDuration:"0.4s",hiddenStyle:{opacity:0,transform:"scale(0.001)"},visibleStyle:{opacity:1,transform:"scale(1)"}};var f=r.prototype;n.extend(f,e.prototype),f.option=function(t){n.extend(this.options,t)},f._getOption=function(t){var e=this.constructor.compatOptions[t];return e&&void 0!==this.options[e]?this.options[e]:this.options[t]},r.compatOptions={initLayout:"isInitLayout",horizontal:"isHorizontal",layoutInstant:"isLayoutInstant",originLeft:"isOriginLeft",originTop:"isOriginTop",resize:"isResizeBound",resizeContainer:"isResizingContainer"},f._create=function(){this.reloadItems(),this.stamps=[],this.stamp(this.options.stamp),n.extend(this.element.style,this.options.containerStyle);var t=this._getOption("resize");t&&this.bindResize()},f.reloadItems=function(){this.items=this._itemize(this.element.children)},f._itemize=function(t){for(var e=this._filterFindItemElements(t),i=this.constructor.Item,n=[],o=0;o<e.length;o++){var r=e[o],s=new i(r,this);n.push(s)}return n},f._filterFindItemElements=function(t){return n.filterFindElements(t,this.options.itemSelector)},f.getItemElements=function(){return this.items.map(function(t){return t.element})},f.layout=function(){this._resetLayout(),this._manageStamps();var t=this._getOption("layoutInstant"),e=void 0!==t?t:!this._isLayoutInited;this.layoutItems(this.items,e),this._isLayoutInited=!0},f._init=f.layout,f._resetLayout=function(){this.getSize()},f.getSize=function(){this.size=i(this.element)},f._getMeasurement=function(t,e){var n,o=this.options[t];o?("string"==typeof o?n=this.element.querySelector(o):o instanceof HTMLElement&&(n=o),this[t]=n?i(n)[e]:o):this[t]=0},f.layoutItems=function(t,e){t=this._getItemsForLayout(t),this._layoutItems(t,e),this._postLayout()},f._getItemsForLayout=function(t){return t.filter(function(t){return!t.isIgnored})},f._layoutItems=function(t,e){if(this._emitCompleteOnItems("layout",t),t&&t.length){var i=[];t.forEach(function(t){var n=this._getItemLayoutPosition(t);n.item=t,n.isInstant=e||t.isLayoutInstant,i.push(n)},this),this._processLayoutQueue(i)}},f._getItemLayoutPosition=function(){return{x:0,y:0}},f._processLayoutQueue=function(t){this.updateStagger(),t.forEach(function(t,e){this._positionItem(t.item,t.x,t.y,t.isInstant,e)},this)},f.updateStagger=function(){var t=this.options.stagger;return null===t||void 0===t?void(this.stagger=0):(this.stagger=a(t),this.stagger)},f._positionItem=function(t,e,i,n,o){n?t.goTo(e,i):(t.stagger(o*this.stagger),t.moveTo(e,i))},f._postLayout=function(){this.resizeContainer()},f.resizeContainer=function(){var t=this._getOption("resizeContainer");if(t){var e=this._getContainerSize();e&&(this._setContainerMeasure(e.width,!0),this._setContainerMeasure(e.height,!1))}},f._getContainerSize=d,f._setContainerMeasure=function(t,e){if(void 0!==t){var i=this.size;i.isBorderBox&&(t+=e?i.paddingLeft+i.paddingRight+i.borderLeftWidth+i.borderRightWidth:i.paddingBottom+i.paddingTop+i.borderTopWidth+i.borderBottomWidth),t=Math.max(t,0),this.element.style[e?"width":"height"]=t+"px"}},f._emitCompleteOnItems=function(t,e){function i(){o.dispatchEvent(t+"Complete",null,[e])}function n(){s++,s==r&&i()}var o=this,r=e.length;if(!e||!r)return void i();var s=0;e.forEach(function(e){e.once(t,n)})},f.dispatchEvent=function(t,e,i){var n=e?[e].concat(i):i;if(this.emitEvent(t,n),u)if(this.$element=this.$element||u(this.element),e){var o=u.Event(e);o.type=t,this.$element.trigger(o,i)}else this.$element.trigger(t,i)},f.ignore=function(t){var e=this.getItem(t);e&&(e.isIgnored=!0)},f.unignore=function(t){var e=this.getItem(t);e&&delete e.isIgnored},f.stamp=function(t){t=this._find(t),t&&(this.stamps=this.stamps.concat(t),t.forEach(this.ignore,this))},f.unstamp=function(t){t=this._find(t),t&&t.forEach(function(t){n.removeFrom(this.stamps,t),this.unignore(t)},this)},f._find=function(t){return t?("string"==typeof t&&(t=this.element.querySelectorAll(t)),t=n.makeArray(t)):void 0},f._manageStamps=function(){this.stamps&&this.stamps.length&&(this._getBoundingRect(),this.stamps.forEach(this._manageStamp,this))},f._getBoundingRect=function(){var t=this.element.getBoundingClientRect(),e=this.size;this._boundingRect={left:t.left+e.paddingLeft+e.borderLeftWidth,top:t.top+e.paddingTop+e.borderTopWidth,right:t.right-(e.paddingRight+e.borderRightWidth),bottom:t.bottom-(e.paddingBottom+e.borderBottomWidth)}},f._manageStamp=d,f._getElementOffset=function(t){var e=t.getBoundingClientRect(),n=this._boundingRect,o=i(t),r={left:e.left-n.left-o.marginLeft,top:e.top-n.top-o.marginTop,right:n.right-e.right-o.marginRight,bottom:n.bottom-e.bottom-o.marginBottom};return r},f.handleEvent=n.handleEvent,f.bindResize=function(){t.addEventListener("resize",this),this.isResizeBound=!0},f.unbindResize=function(){t.removeEventListener("resize",this),this.isResizeBound=!1},f.onresize=function(){this.resize()},n.debounceMethod(r,"onresize",100),f.resize=function(){this.isResizeBound&&this.needsResizeLayout()&&this.layout()},f.needsResizeLayout=function(){var t=i(this.element),e=this.size&&t;return e&&t.innerWidth!==this.size.innerWidth},f.addItems=function(t){var e=this._itemize(t);return e.length&&(this.items=this.items.concat(e)),e},f.appended=function(t){var e=this.addItems(t);e.length&&(this.layoutItems(e,!0),this.reveal(e))},f.prepended=function(t){var e=this._itemize(t);if(e.length){var i=this.items.slice(0);this.items=e.concat(i),this._resetLayout(),this._manageStamps(),this.layoutItems(e,!0),this.reveal(e),this.layoutItems(i)}},f.reveal=function(t){if(this._emitCompleteOnItems("reveal",t),t&&t.length){var e=this.updateStagger();t.forEach(function(t,i){t.stagger(i*e),t.reveal()})}},f.hide=function(t){if(this._emitCompleteOnItems("hide",t),t&&t.length){var e=this.updateStagger();t.forEach(function(t,i){t.stagger(i*e),t.hide()})}},f.revealItemElements=function(t){var e=this.getItems(t);this.reveal(e)},f.hideItemElements=function(t){var e=this.getItems(t);this.hide(e)},f.getItem=function(t){for(var e=0;e<this.items.length;e++){var i=this.items[e];if(i.element==t)return i}},f.getItems=function(t){t=n.makeArray(t);var e=[];return t.forEach(function(t){var i=this.getItem(t);i&&e.push(i)},this),e},f.remove=function(t){var e=this.getItems(t);this._emitCompleteOnItems("remove",e),e&&e.length&&e.forEach(function(t){t.remove(),n.removeFrom(this.items,t)},this)},f.destroy=function(){var t=this.element.style;t.height="",t.position="",t.width="",this.items.forEach(function(t){t.destroy()}),this.unbindResize();var e=this.element.outlayerGUID;delete c[e],delete this.element.outlayerGUID,u&&u.removeData(this.element,this.constructor.namespace)},r.data=function(t){t=n.getQueryElement(t);var e=t&&t.outlayerGUID;return e&&c[e]},r.create=function(t,e){var i=s(r);return i.defaults=n.extend({},r.defaults),n.extend(i.defaults,e),i.compatOptions=n.extend({},r.compatOptions),i.namespace=t,i.data=r.data,i.Item=s(o),n.htmlInit(i,t),u&&u.bridget&&u.bridget(t,i),i};var m={ms:1,s:1e3};return r.Item=o,r}),function(t,e){"function"==typeof define&&define.amd?define(["outlayer/outlayer","get-size/get-size"],e):"object"==typeof module&&module.exports?module.exports=e(require("outlayer"),require("get-size")):t.Masonry=e(t.Outlayer,t.getSize)}(window,function(t,e){var i=t.create("masonry");i.compatOptions.fitWidth="isFitWidth";var n=i.prototype;return n._resetLayout=function(){this.getSize(),this._getMeasurement("columnWidth","outerWidth"),this._getMeasurement("gutter","outerWidth"),this.measureColumns(),this.colYs=[];for(var t=0;t<this.cols;t++)this.colYs.push(0);this.maxY=0,this.horizontalColIndex=0},n.measureColumns=function(){if(this.getContainerWidth(),!this.columnWidth){var t=this.items[0],i=t&&t.element;this.columnWidth=i&&e(i).outerWidth||this.containerWidth}var n=this.columnWidth+=this.gutter,o=this.containerWidth+this.gutter,r=o/n,s=n-o%n,a=s&&1>s?"round":"floor";r=Math[a](r),this.cols=Math.max(r,1)},n.getContainerWidth=function(){var t=this._getOption("fitWidth"),i=t?this.element.parentNode:this.element,n=e(i);this.containerWidth=n&&n.innerWidth},n._getItemLayoutPosition=function(t){t.getSize();var e=t.size.outerWidth%this.columnWidth,i=e&&1>e?"round":"ceil",n=Math[i](t.size.outerWidth/this.columnWidth);n=Math.min(n,this.cols);for(var o=this.options.horizontalOrder?"_getHorizontalColPosition":"_getTopColPosition",r=this[o](n,t),s={x:this.columnWidth*r.col,y:r.y},a=r.y+t.size.outerHeight,h=n+r.col,u=r.col;h>u;u++)this.colYs[u]=a;return s},n._getTopColPosition=function(t){var e=this._getTopColGroup(t),i=Math.min.apply(Math,e);return{col:e.indexOf(i),y:i}},n._getTopColGroup=function(t){if(2>t)return this.colYs;for(var e=[],i=this.cols+1-t,n=0;i>n;n++)e[n]=this._getColGroupY(n,t);return e},n._getColGroupY=function(t,e){if(2>e)return this.colYs[t];var i=this.colYs.slice(t,t+e);return Math.max.apply(Math,i)},n._getHorizontalColPosition=function(t,e){var i=this.horizontalColIndex%this.cols,n=t>1&&i+t>this.cols;i=n?0:i;var o=e.size.outerWidth&&e.size.outerHeight;return this.horizontalColIndex=o?i+t:this.horizontalColIndex,{col:i,y:this._getColGroupY(i,t)}},n._manageStamp=function(t){var i=e(t),n=this._getElementOffset(t),o=this._getOption("originLeft"),r=o?n.left:n.right,s=r+i.outerWidth,a=Math.floor(r/this.columnWidth);a=Math.max(0,a);var h=Math.floor(s/this.columnWidth);h-=s%this.columnWidth?0:1,h=Math.min(this.cols-1,h);for(var u=this._getOption("originTop"),d=(u?n.top:n.bottom)+i.outerHeight,l=a;h>=l;l++)this.colYs[l]=Math.max(d,this.colYs[l])},n._getContainerSize=function(){this.maxY=Math.max.apply(Math,this.colYs);var t={height:this.maxY};return this._getOption("fitWidth")&&(t.width=this._getContainerFitWidth()),t},n._getContainerFitWidth=function(){for(var t=0,e=this.cols;--e&&0===this.colYs[e];)t++;return(this.cols-t)*this.columnWidth-this.gutter},n.needsResizeLayout=function(){var t=this.containerWidth;return this.getContainerWidth(),t!=this.containerWidth},i});
