/* © brix cross media AG, http://www.brix.ch - It is strictly forbidden to change, reuse or copy the source code of this document. */
;(function ( $, window, document, undefined ) {
    var pluginName = "anuraSearch",
        defaults = {
            server: 'https://celum.local/anura/first', // REQUIRED, which anura endpoint to use
            ext: '.do?', // extension of server pages (.php, .do)
            locale: '', // ISO 639-1 code, blank for default
            widgets: [ // which search controls to show and in what order
                {name: 'text', scope: null, checked: false}, // scope needs an anuraTree of folders or keywords, e.g. $('#navigation')
                {name: 'filetype'} // document type (checkboxes with images, documents, audio, video, others)
                //{name: 'assettype'} // document type (checkboxes with images, documents, audio, video, others)
                //{name: 'keywords', root: <keyword id>, multiple: <bool>, show: <keyword id>, autosubmit: <bool>, unselectable: <bool>} // requires jquery.anura.tree
                //{name: 'infofield', id: <infofield id>, style: dropdown/select/checkbox/radio/tree, collapsible: false, collapsed: false}
                //{name: 'properties', uploaded: true, modified: true, size: true}
                //{name: 'submit', show: true, resettable: false}
            ],
            callback: function(server, search_query) { // what do do with the search result
                $('.anura-gallery-container').first().parent().anuraGallery({server: server, search: {custom: search_query}});
            },
            datetimepicker: {format:'Y-m-d', validateOnBlur: false, timepicker: false, onSelectDate: function(dp, input) {input.change();}},
            infostate_transform: null, // mess with infofield states before the widget is loaded, e.g. function(widget, infoState) {return whatever;};
            chosen: {display_selected_options: false, search_contains: true, allow_single_deselect: true, placeholder_text_multiple: ' ', placeholder_text_single: ' ', disable_search_threshold: 6 /* nr. of file types ^^*/},
            folder: 0, // DEPRECATED (use custom: 'search_node=...') restrict search to this folder - show.scope overrides this if enabled
            keyword: 0, // DEPRECATED (use custom: 'search_node=...') restrict search to this keyword - show.scope or keywords override this if enabled
            custom: '', // arbitrary arguments sent along with the request, i.e. search_infofield=628,1&search_node=123
            resettable: true, // DEPRECATED (use the widget's setting) adds a reset link to the bottom
            state: '', // prefix for state in url (multi-instance) or false to disable state
            container_class: '', // additional class for each widget container (may be overridden in widget settings)
            button_class: '', // additional class applied to all buttons (e.g. "btn btn-default" when used with bootstrap)
            initial_search: true, // triggers an "empty" search when anuraSearch is done loading to show "all" assets
            fallback_search: null, // what to search for when no search was entered by the user (initially or after reset) - leave blank to search for everything
            facets: false, // request faceted response when searching
            facet_count: true // whether to show the available assets on a infofield value, e.g. "Images [1337]" instead of just "Images"
        };
    function Plugin( element, options ) {
        this.element = this.container = $(element);
        this.options = $.extend( {}, defaults, options );
        if (this.options.datetimepicker) this.options.datetimepicker = $.extend(this.options.datetimepicker, {lang: this.options.locale?this.options.locale:'en'});
        this._defaults = defaults;
        this._name = pluginName;
        this.strings = {};
        this.facet_fields = [];
        this.rpc_count = 0;
        this.init();
    }
    Plugin.prototype = {
        init: function() {
            var _this = this;
            $.anura.getJSON(this, this.options.server+'/asset'+this.options.ext+'callback=?&string='+
                ['search.startSearch', 'search.searchItem', 'general.all', 'search.restrictFolder', 'general.keywords', 'search.addButton', 'lookup.addKeywordButton',
                    'common.object.technicaltype', 'general.images', 'general.documents', 'general.videos', 'general.audios', 'search.searchOtherObjects', 'search.cleanUp',
                    'search.documentProperties', 'search.uploaded', 'search.modified', 'AssetProperties.width', 'AssetProperties.height', 'anura.searchOnlyIn',
                    'imagemanagement.thumblist.searchResultEmpty', 'anura.searchNoResult', 'common.object.mediatype', 'common.object.fileExtension'].join('&string='),
                {locale: this.options.locale}, 'anuraSearchStrings', function(data) {
                    _this.strings = data;
                    _this.populate();
                });
        },
        populate: function() {
            this.element.empty();
            this.rpc_count = 0;
            this.container = $('<div />').addClass('anura-search-container anura-loading').appendTo(this.element)
                .removeClass(function (index, css) {
                    return (css.match(/anura-locale-\S+/g) || []).join(' ');
                }).addClass('anura-locale-' + this.options.locale);
            if (this.options.facet_count === false) this.container.addClass('anura-hide-facet-count');
            var _this = this, id_ext = this.element.attr('id') ? ('-' + this.element.attr('id')) : ''; // multi-instance
            $('<div/>').addClass('anura-loader').appendTo($('<div/>').addClass('input-blocker').appendTo(this.container));
            var has_submit = false;
            $.each(this.options.widgets, function () {
                if ($.isFunction(this)) return this(_this.container, id_ext);
                switch (this.name) {
                    case 'text':
                        _this.createTextWidget(this, id_ext);
                        break;
                    case 'type':            // legacy name of filetype
                    case 'filetype':
                    case 'file_categories':
                        _this.createFileTypeWidget(this, id_ext);
                        break;
                    case 'assettype':
                        _this.createAssetTypeWidget(this, id_ext);
                        break;
                    case 'properties':
                        _this.createPropsWidget(this, id_ext);
                        break;
                    case 'keywords': // legacy
                    case 'tree':
                        _this.createTreeWidget(this, id_ext);
                        break;
                    case 'infofield':
                        _this.createInfofieldWidget(this, id_ext);
                        break;
                    case 'extensions':
                        _this.createExtensionsWidget(this, id_ext);
                        break;
                    case 'id':
                        _this.createIdWidget(this, id_ext);
                        break;
                    case 'submit':
                        _this.createSubmitWidget(this, id_ext);
                        has_submit = true;
                        break;
                    default:
                        $('<p/>').text('Invalid configuration: ' + JSON.stringify(this)).appendTo(this.container);
                }
            });
            if (!has_submit) _this.createSubmitWidget({show: true});
            if (this.rpc_count < 1) this.rpcDone('populate');

            if (jQuery().chosen) $(window).on('resize', this.resizeChosen);
        },
        doSearch: function() {
            this.container.addClass('anura-loading');
            var _this = this, args = {search_keyword: [], search_infofield: []};
        // TODO each widget should provide its own search params, so only it has to know how it works
            if (this.options.keyword) args.search_keyword.push(this.options.keyword);
            else if (this.options.folder) args.search_folder = this.options.folder;
        // text
            var search_text = this.container.find('.anura-search-text');
            if (search_text.length) args.search = search_text.val();
            $.anura.pushState(this.options.state, 'search-text', args.search);
        // scope
            var scope_cb = this.container.find('.anura-search-scope input[type=checkbox]');
            if (scope_cb.length) {
                $.anura.pushState(this.options.state, 'search-scope', scope_cb.is(':checked'));
                if (scope_cb.is(':checked') && scope_cb.data('id')) args.search_node = scope_cb.data('id');
            }
        // file type
            var searchTypeOn = [],  searchTypeOff = [], searchTypeUrl = [];
            var dynamicFileCat = this.container.find('.anura-search-type.anura-dynamic-file-categories').length > 0;
            this.container.find('.anura-search-type input[type=checkbox], .anura-search-type option').each(function() {
                var typeInput = $(this);
                if (!typeInput.attr('value')) return; // "all" dropdown option
                if ((typeInput.is(':checked') || typeInput.is(':selected')) && !typeInput.is(':disabled')) {
                    searchTypeOn.push($(this).attr('value'));
                    searchTypeUrl.push(typeInput.attr('value').replace('search_', '').substr(0, 1));
                } else {
                    searchTypeOff.push($(this).attr('value'));
                }
            });
            if (dynamicFileCat) args['search_file_category'] = searchTypeOn.join(',');
            else if (searchTypeOn.length > 0 && searchTypeOn.length < 5) $.each(searchTypeOff, function (i, t) {args[t] = false;});
            $.anura.pushState(_this.options.state, 'search-type', searchTypeUrl.length?searchTypeUrl.join(','):null);
        // asset type
            var searchAssetType = [];
            this.container.find('.anura-search-assettype input, .anura-search-assettype option').each(function() {
                var type = $(this);
                if (!type.attr('value')) return; // "all" dropdown option
                if ((type.is(':checked') || type.is(':selected')) && !type.is(':disabled')) {
                    searchAssetType.push($(this).attr('value'));
            }
            });
            if (searchAssetType.length > 0) args['search_asset_type'] = searchAssetType.join(',');
            $.anura.pushState(_this.options.state, 'search-assettype', searchAssetType.length?searchAssetType.join(','):null);
        // asset id
            var asset_ids = this.container.find('.anura-search-id input').val();
            if (asset_ids) {
                asset_ids = asset_ids.split(/[\D*]/).join(','); // normalize spaces, commas etc.
                args['search_id'] = asset_ids;
                $.anura.pushState(_this.options.state, 'search-id', asset_ids);
            } else {
                $.anura.removeState(_this.options.state, 'search-id');
            }
        // file extension
            var file_extensions = this.container.find('.anura-search-extensions input').val();
            if (file_extensions) {
                file_extensions = file_extensions.split(/[, ]+/).join(','); // normalize spaces, commas etc.
                args['search_extensions'] = file_extensions;
                $.anura.pushState(_this.options.state, 'search-extensions', file_extensions);
            } else {
                $.anura.removeState(_this.options.state, 'search-extensions');
            }
        // properties
            var prop_input = this.container.find('.anura-search-properties input.uploaded.from');
            if (prop_input.length) args.search_uploaded_after = prop_input.val();
            $.anura.pushState(_this.options.state, 'search-uploaded-after', args.search_uploaded_after);
            prop_input = this.container.find('.anura-search-properties input.uploaded.to');
            if (prop_input.length) args.search_uploaded_before = prop_input.val();
            $.anura.pushState(_this.options.state, 'search-uploaded-before', args.search_uploaded_before);
            prop_input = this.container.find('.anura-search-properties input.modified.from');
            if (prop_input.length) args.search_modified_after = prop_input.val();
            $.anura.pushState(_this.options.state, 'search-modified-after', args.search_modified_after);
            prop_input = this.container.find('.anura-search-properties input.modified.to');
            if (prop_input.length) args.search_modified_before = prop_input.val();
            $.anura.pushState(_this.options.state, 'search-modified-before', args.search_modified_before);
            prop_input = this.container.find('.anura-search-properties input.width.from');
            if (prop_input.length) args.search_min_width = prop_input.val();
            prop_input = this.container.find('.anura-search-properties input.width.to');
            if (prop_input.length) args.search_max_width = prop_input.val();
            prop_input = this.container.find('.anura-search-properties input.height.from');
            if (prop_input.length) args.search_min_height = prop_input.val();
            prop_input = this.container.find('.anura-search-properties input.height.to');
            if (prop_input.length) args.search_max_height = prop_input.val();
        // keywords
            var keywords_tree = this.container.find('.anura-search-keyword-tree');
            if (keywords_tree.length) {
                keywords_tree.each(function () {
                    var selected = $(this).data('selected');
                    if (selected) args.search_keyword.push(selected);
                });
                this.container.find('.anura-search-keyword-holder > li').each(function() {
                    if ($(this).data('id')) args.search_keyword.push($(this).data('id'));
                });
            }
        // infofields
            var infofield_search = this.container.find('.anura-search-infofield');
            $.each(infofield_search, function() {
                var info = $(this), selectVal = info.find('select').val(), noderef = info.find('.anura-search-noderef');
                if (selectVal && selectVal.length) {
                    if ($.isArray(selectVal)) selectVal = selectVal.join(noderef.data('and-mode')?'+':',');
                    if (selectVal.indexOf(' ') >= 0) selectVal = '"'+selectVal+'"';
                    if (noderef.data('recursive') !== undefined) selectVal += (noderef.data('recursive')?',1':',0');
                    args.search_infofield.push(info.data('id')+','+selectVal);
                    $.anura.pushState(_this.options.state, 'search-info-'+info.data('id'), selectVal);
                } else if (info.find('input[type=text], input[type=number], input[type=date]').length > 0) {
                    if (info.find('.range').length === 0) {
                        var val = info.find('input.text').val();
                    } else {
                        var from = info.find('input.from').val() || '';
                        var to = info.find('input.to').val() || '';
                        var val = from.replace(/,/g,'.')+((from || to)?',':'')+to.replace(/,/g,'.');
                    }
                    if (val) {
                        if (info.find('.anura-search-noderef').length !== 0) val = '"'+val+'"';
                        args.search_infofield.push(info.data('id')+','+val);
                        $.anura.pushState(_this.options.state, 'search-info-'+info.data('id'), val);
                    } else {
                        $.anura.removeState(_this.options.state, 'search-info-'+info.data('id'));
                    }
                } else if (info.find('input[type=radio]').is(':checked')) {
                    var checked = info.find('input[type=radio]:checked').val();
                    if (checked !== undefined) args.search_infofield.push(info.data('id')+','+checked);
                    $.anura.pushState(_this.options.state, 'search-info-'+info.data('id'), checked);
                }  else if (info.find('input[type=checkbox]').is(':checked')) {
                    var checked = info.find('input[type=checkbox]:checked').map(function(){return $(this).val();}).toArray();
                    var val = (checked.length > 0)?checked.join(','):null;
                    if (val) args.search_infofield.push(info.data('id')+','+val);
                    $.anura.pushState(_this.options.state, 'search-info-'+info.data('id'), val);
                } else if (noderef.data('selected')) {
                    var selected = noderef.data('selected');
                    if ($.isArray(selected)) selected = selected.join(noderef.data('and-mode')?'+':',');
                    if (selected) args.search_infofield.push(info.data('id')+','+selected+','+(noderef.data('recursive')?'1':'0'));
                } else {
                    $.anura.removeState(_this.options.state, 'search-info-'+info.data('id'));
                }
            });

            this.container.find('.anura-search-message').remove();
            var customParam = '';
            if (this.options.custom) {
                if ($.isFunction(this.options.custom)) customParam = this.options.custom();
                else customParam = this.options.custom;
                if (!customParam) customParam = ''; // callback sent crap
                if (customParam.length > 0 && customParam[0] !== '&') customParam = '&'+customParam;
            }
            var searchParams = $.param(args).replace(/%5B%5D/g, '').replace(/%2C/g, ',')+customParam;
            try {
                if (this.options.fallback_search && (!searchParams || searchParams === 'search='+customParam)) searchParams = this.options.fallback_search;
                this.options.callback(this.options.server, searchParams);
            } catch (e) {
                if (console) console.log("Callback failed", e);
            }
        // faceted search
            if (this.options.facets) {
                this.container.addClass('anura-loading');
                $.anura.getJSON(this, this.options.server+'/node'+_this.options.ext+'callback=?&search_facets=1&'+
                    'facet_fields='+this.facet_fields.join(',')+'&'+(searchParams === this.options.fallback_search ?
                        (this.options.custom ? this.options.custom : 'custom=') : searchParams), {},
                            'anuraSearchFacet', function (data) {
                    if ($.isEmptyObject(data)) console.warn('received empty facet response, ignoring (check your solr setup)')
                    else _this.container.trigger('anura-facet-update', [data]);
                    _this.container.removeClass('anura-loading');
                }, function () { // on fail
                    _this.container.removeClass('anura-loading');
                });
            } else {
                this.container.removeClass('anura-loading');
            }
        },
        rpcStart: function (src) {
            this.rpc_count += 1;
            //console.log(src+' start ('+this.rpc_count+')');
        },
        rpcDone: function (src) {
            this.rpc_count -= 1;
            //console.log(src+' stop ('+this.rpc_count+')');
            if (this.rpc_count < 1) {
                if (this.rpc_count < 0 && console) console.log('Late RPC done: '+src+' ('+this.rpc_count+')');
                this.element.trigger('anura-init-search', this.options);
                if (this.options.initial_search || window.location.hash.indexOf('search-') > 0) {
                    this.options.initial_search = false;
                    this.doSearch();
                }
                if (jQuery().chosen) this.resizeChosen();
                this.container.removeClass('anura-loading');
            }
        },
        createSubmitWidget: function(widget, id_ext) {
            var _this = this;
            var submit_container = $('<div />').addClass('anura-search-submit '+this.options.container_class).appendTo(this.container);
            if (!widget.show) submit_container.hide();
            $('<input type="button" value="'+_this.strings['search.startSearch']+'">').addClass('anura-search-button '+this.options.button_class)
                .on('click', function() {_this.doSearch();}).appendTo(submit_container);
            if (widget.resettable || this.options.resettable) $('<input type="button" value="'+_this.strings['search.cleanUp']+'">').on('click', function() {
                _this.resetSearch();
                _this.doSearch();
            }).addClass('anura-search-reset '+this.options.button_class).appendTo(submit_container);
            this.element.on('reset', function (e, o) {
                _this.resetSearch();
                if (o) _this.doSearch();
            });
        },
        resetSearch: function() { // TODO this should be done by each widget itself
            this.container.find('input').not('[type=button]').prop('checked', false);
            this.container.find('select').val('').trigger('chosen:updated');
            this.container.find('input[type=text], input[type=number], input[type=date]').val(null);
            this.container.find('.anura-search-type input, .anura-search-assettype input').prop('checked', false);
            var scope = this.container.find('.anura-search-scope input');
            if (scope) scope.prop('checked', scope.data('default-checked')).data('id', null).find('anura-value').text('');
            this.container.find('.anura-search-keyword-holder li').remove();
            this.container.find('.anura-search-keyword-tree').data('selected', null).find('li').removeClass('anura-active');
            this.container.find('.anura-search-keyword-tree ul .anura-tree-unfolded').addClass('anura-tree-folded').removeClass('anura-tree-unfolded');
            this.container.find('.anura-search-noderef')
                .data('selected', null).data('selected_name', null)
                .find('.anura-tree-container').trigger('anura-reset')
                .find('.anura-active, .anura-current').removeClass('anura-active anura-current');
            this.container.find('.anura-search-noderef ul .anura-tree-unfolded').addClass('anura-tree-folded').removeClass('anura-tree-unfolded');
            this.container.find('.anura-search-noderef').parent().find('legend > span').text('');
            this.container.find('.anura-search-default-not-collapsed').removeClass('anura-search-collapsed');
            this.container.find('.anura-search-default-collapsed').addClass('anura-search-collapsed');
            this.container.find('.anura-search-id input').val(null);
            $.anura.removeStates(this.options.state, 'search');
            this.element.trigger('anura-reset', this.options);
        },
        createTextWidget: function(widget, id_ext) {
            this.facet_fields.push('text');
            var _this = this, container_class = widget.container_class?widget.container_class:this.options.container_class;
            var text_container = $('<div />').addClass('anura-search-text-container '+container_class).appendTo(this.container);
            if (widget.title) $('<legend />').text(widget.title[this.options.locale]?widget.title[this.options.locale]:widget.title).appendTo(text_container);
            var search_input = $('<input type="text" placeholder="' + this.strings['search.searchItem'] + '">').addClass('anura-search-text')
                .val($.anura.getState(this.options.state, 'search-text', '').replace(/\+/g, ' ')).appendTo(text_container)
                .on('keypress', function (e) {if (e.keyCode === 13) _this.doSearch();}).on('change', function (e) {_this.doSearch();});
            if (widget.resettable) {
                $('<span/>').addClass('anura-search-text-reset').appendTo(text_container.addClass('anura-search-text-has-reset'))
                    .on('click', function () {search_input.val('').change();});
            }
            if (widget.button) {
                $('<button/>').addClass('anura-search-text-button').appendTo(text_container.addClass('anura-search-text-has-button'))
                    .on('click', function () {_this.doSearch();});
            }
            if (widget.scope) {
                var scope_container = $('<div />').addClass('anura-search-scope').appendTo(text_container);
                var scope_state = $.anura.getState(this.options.state, 'search-scope', widget.checked);
                var scope_input = $('<input type="checkbox" id="anura-search-scope' + id_ext + '">').appendTo(scope_container)
                    .prop('checked', scope_state).data('default-checked', scope_state);
                var scope_label_text = (this.strings['anura.searchOnlyIn'] === 'anura.searchOnlyIn') ? this.strings['search.restrictFolder'] : this.strings['anura.searchOnlyIn'];
                var scope_label = $('<label for="anura-search-scope' + id_ext + '"><span class="anura-name">' + scope_label_text
                    + ': </span><span class="anura-value"></span></label>').appendTo(scope_container);
                $('<div style="clear: both;">').appendTo(scope_container);
                var update_scope = function (event, server, kind, asset_id, name) {
                    scope_input.parent().show();
                    scope_input.data('kind', kind).data('id', asset_id);
                    scope_label.find('.anura-value').text(name);
                    if (widget.autosubmit) _this.doSearch();
                };
                widget.scope.on('anura-clicked', update_scope);
                update_scope(null, null, widget.scope.data('kind'), widget.scope.data('selected'), widget.scope.data('selected_name'));
            }
            this.element.on('set-search-text', function (e, o) {
                search_input.val(o);
                _this.doSearch();
            })
        },
        createFileTypeWidget: function(widget, id_ext) {
            this.facet_fields.push('assetType');
            var container_class = widget.container_class?widget.container_class:this.options.container_class, _this = this;
            var field_container = $('<div />').addClass('anura-field-container '+container_class).appendTo(this.container);
            var fieldset = $('<fieldset />').addClass('anura-search-type').appendTo(field_container);
            var typeStates = $.anura.getState(this.options.state, 'search-type', '');
            if (typeStates.length) typeStates = typeStates.split(',');
            $('<legend>' + ((widget.title)?(widget.title[this.options.locale]?widget.title[this.options.locale]:widget.title):this.strings['common.object.technicaltype']) + '</legend>').appendTo(fieldset);
            var items = [];
            var createDom = function() {
                if (widget.style === 'select' || widget.style === 'dropdown') {
                    var select = $('<select/>').addClass('anura-search-type-select').appendTo(fieldset);
                    if (widget.style === 'select') select.prop('multiple', true);
                    else $('<option value=""/>').text(_this.strings['general.all']).appendTo(select).prop('selected', true);
                    $.each(items, function(index, item) {
                        $('<option data-nr="'+item.nr+'"/>').attr('value', item.value).text(item.name).appendTo(select).prop('selected', item.selected);
                    });
                    if (widget.autosubmit) select.on('change', function() {_this.doSearch();});
                    if (jQuery().chosen) select.chosen(_this.options.chosen);
                } else {
                    $.each(items, function(index, item) {
                        var uid = 'anura-'+item.value.replace('_', '-')+ id_ext;
                        var typediv = $('<div/>').addClass('anura-type-'+item.value.replace('_', '-')).appendTo(fieldset);
                        var cb = $('<input type="'+(widget.style === 'radio'?'radio':'checkbox')+'" data-nr="'+item.nr+'"/>').attr('id', uid).attr('value', item.value).prop('checked', item.selected).appendTo(typediv);
                        var label = $('<label data-nr="'+item.nr+'"/>').attr('for', uid).text(item.name).appendTo(typediv);
                        if (_this.options.facets) $('<span>').addClass('anura-asset-count').appendTo(label);
                        if (widget.autosubmit) cb.on('change', function() {_this.doSearch();});
                    });
                }
            }
            $.anura.getJSON(this, this.options.server+'/asset'+_this.options.ext+'callback=?&file_categories=true', {locale: this.options.locale}, 'anuraSearchFileCat',
            function (data) {
                $.each(data, function (index, item) {
                    items.push({value: item.id, nr: item.id, name: item.name,
                        selected: typeStates.length ? $.inArray(item.id, typeStates) >= 0 : $.inArray(item.id, widget.selected) >= 0});
                });
                createDom();
                fieldset.addClass('anura-dynamic-file-categories');
            }, function () { // legacy, hard-coded values
                items.push({value: 'search_images',    nr: 1, name: _this.strings['general.images'], selected: typeStates.length ? $.inArray('i', typeStates) >= 0 : widget.images});
                items.push({value: 'search_documents', nr: 2, name: _this.strings['general.documents'], selected: typeStates.length ? $.inArray('d', typeStates) >= 0 : widget.documents});
                items.push({value: 'search_videos',    nr: 3, name: _this.strings['general.videos'], selected: typeStates.length ? $.inArray('v', typeStates) >= 0 : widget.videos});
                items.push({value: 'search_audios',    nr: 4, name: _this.strings['general.audios'], selected: typeStates.length ? $.inArray('a', typeStates) >= 0 : widget.audios});
                items.push({value: 'search_others',    nr: 0, name: _this.strings['search.searchOtherObjects'], selected: typeStates.length ? $.inArray('o', typeStates) >= 0 : widget.others});
                createDom();
            });

            this.container.on('anura-facet-update', function(event, data) {
                if (data && data.assetType) {
                    fieldset.find('input[type=checkbox]').prop('disabled', true);
                    fieldset.find('option[data-nr]').prop('disabled', true).addClass('anura-facet-none');
                    fieldset.find('label .anura-asset-count').text(0).closest('div').addClass('anura-facet-none');
                    if (!$.isEmptyObject(data.assetType)) {
                        $.each(data.assetType, function(nr, count) {
                            var option = fieldset.find('option[data-nr='+nr+']');
                            if (option.length) _this.setOptionFacetCount(option.prop('disabled', false).removeClass('anura-facet-none'), count);
                            else fieldset.find('input[data-nr='+nr+']').prop('disabled', false)
                                    .closest('div').removeClass('anura-facet-none').find('.anura-asset-count').text(count);
                        });
                        field_container.removeClass('anura-facet-none');
                    } else {
                        field_container.addClass('anura-facet-none');
                    }
                    if (jQuery().chosen) fieldset.find('select').trigger('chosen:updated');
                }
            });
        },
        createAssetTypeWidget: function(widget, id_ext) {
            this.facet_fields.push('mediaType');
            var container_class = widget.container_class?widget.container_class:this.options.container_class, _this = this;
            var field_container = $('<div />').addClass('anura-field-container '+container_class).appendTo(this.container);
            var fieldset = $('<fieldset />').addClass('anura-search-assettype').appendTo(field_container);
            var typeStates = $.anura.getState(this.options.state, 'search-assettype', '');
            typeStates = typeStates.length ? typeStates.split(',') : [];
            $('<legend>' + ((widget.title)?(widget.title[this.options.locale]?widget.title[this.options.locale]:widget.title):this.strings['common.object.mediatype']) + '</legend>').appendTo(fieldset);

            var select = null;
            if (widget.style === 'select' || widget.style === 'dropdown') {
                select = $('<select/>').addClass('anura-search-assettype-select').appendTo(fieldset);
                if (widget.style === 'select') select.prop('multiple', true);
                else $('<option value=""/>').text(this.strings['general.all']).appendTo(select).prop('selected', true);
                if (widget.autosubmit) select.on('change', function() {_this.doSearch();});
                if (jQuery().chosen) select.chosen(_this.options.chosen);
            }
            this.rpcStart('asset-type');
            $.anura.getJSON(this, this.options.server + '/asset' + this.options.ext + 'callback=?', {assettype: true, locale: _this.options.locale}, 'anuraAssetTypeFilter', function (data) {
                $.each(data, function (index, at) {
                    if (select) {
                        $('<option data-id="'+at.id+'"/>').attr('value', at.id).text(at.name)
                            .appendTo(select).prop('selected', $.inArray(at.id, typeStates) !== -1);
                    } else {
                        var uid = 'anura-assettype-'+at.id;
                        var typediv = $('<div/>').addClass(uid).appendTo(fieldset);
                        var cb = $('<input type="'+(widget.style === 'radio'?'radio':'checkbox')+'" name="anura-assettype" data-id="'+at.id+'"/>').attr('id', uid)
                            .attr('value', at.id).prop('checked', $.inArray(at.id, typeStates) !== -1).appendTo(typediv);
                        var label = $('<label data-id="'+at.id+'"/>').attr('for', uid).text(at.name).appendTo(typediv);
                        if (_this.options.facets) $('<span>').addClass('anura-asset-count').appendTo(label);
                        if (widget.autosubmit) cb.on('change', function() {_this.doSearch();});
                    }
                });
                if (select && jQuery().chosen) select.trigger('chosen:updated');
                _this.rpcDone('asset-type');
            });

            this.container.on('anura-facet-update', function(event, data) {
                if (data && data.mediaType) {
                    fieldset.find('input[type=checkbox]').prop('disabled', true);
                    fieldset.find('option[data-id]').prop('disabled', true).addClass('anura-facet-none');
                    fieldset.find('label .anura-asset-count').text(0).closest('div').addClass('anura-facet-none');
                    if (!$.isEmptyObject(data.mediaType)) {
                        $.each(data.mediaType, function(id, count) {
                            var option = fieldset.find('option[data-id='+id+']');
                            if (option.length) _this.setOptionFacetCount(option.prop('disabled', false).removeClass('anura-facet-none'), count);
                            else fieldset.find('input[data-id='+id+']').prop('disabled', false)
                                .closest('div').removeClass('anura-facet-none').find('.anura-asset-count').text(count);
                        });
                        field_container.removeClass('anura-facet-none');
                    } else {
                        field_container.addClass('anura-facet-none');
                    }
                    if (jQuery().chosen) fieldset.find('select').trigger('chosen:updated');
                }
            });
        },
        createIdWidget: function(widget, id_ext) {
            var container_class = widget.container_class?widget.container_class:this.options.container_class, _this = this;
            var field_container = $('<div />').addClass('anura-field-container '+container_class).appendTo(this.container);
            var fieldset = $('<fieldset />').addClass('anura-search-id').appendTo(field_container);
            var idStates = $.anura.getState(this.options.state, 'search-id', '');
            idStates = idStates.length ? idStates.split(',') : [];
            $('<legend>' + ((widget.title)?(widget.title[this.options.locale]?widget.title[this.options.locale]:widget.title):'ID') + '</legend>').appendTo(fieldset);

            var idInput = $('<input type="text" class="asset-id" title="ID">').val(idStates.join(',')).appendTo(fieldset);
            if (widget.autosubmit) idInput.on('change', function() {_this.doSearch();});
        },
        createExtensionsWidget: function(widget, id_ext) {
            var container_class = widget.container_class?widget.container_class:this.options.container_class, _this = this;
            var field_container = $('<div />').addClass('anura-field-container '+container_class).appendTo(this.container);
            var fieldset = $('<fieldset />').addClass('anura-search-extensions').appendTo(field_container);
            var extStates = $.anura.getState(this.options.state, 'search-extensions', '');
            extStates = extStates.length ? extStates.split(',') : [];
            $('<legend>' + ((widget.title)?(widget.title[this.options.locale]?widget.title[this.options.locale]:widget.title):_this.strings['common.object.fileExtension']) + '</legend>').appendTo(fieldset);

            var extInput = $('<input type="text" class="file-extensions" title="'+_this.strings['common.object.fileExtension']+'">').val(extStates.join(',')).appendTo(fieldset);
            if (widget.autosubmit) extInput.on('change', function() {_this.doSearch();});
        },
        createPropsWidget: function(widget, id_ext) {
            var container_class = widget.container_class?widget.container_class:this.options.container_class;
            var field_container = $('<div />').addClass('anura-field-container '+container_class).appendTo(this.container);
            var item = $('<fieldset />').addClass('anura-search-properties').appendTo(field_container), _this = this;
            $('<legend>' + ((widget.title)?(widget.title[this.options.locale]?widget.title[this.options.locale]:widget.title):this.strings['search.documentProperties']) + '</legend>').appendTo(item);
            if (widget.uploaded !== false) {
                this.facet_fields.push('fileProperties_date_uploaded');
                $('<p />').text(this.strings['search.uploaded'] + ':').appendTo(item);
                $('<input type="date" class="from uploaded datepicker">').appendTo(item).val($.anura.getState(this.options.state, 'search-uploaded-after', ''));
                $('<span class="range"> - </span>').appendTo(item);
                $('<input type="date" class="to uploaded datepicker">').appendTo(item).val($.anura.getState(this.options.state, 'search-uploaded-before', ''));
            }
            if (widget.modified !== false) {
                this.facet_fields.push('fileProperties_date_modified');
                $('<p />').text(this.strings['search.modified'] + ':').appendTo(item);
                $('<input type="date" class="from modified datepicker">').appendTo(item).val($.anura.getState(this.options.state, 'search-modified-after', ''));
                $('<span class="range"> - </span>').appendTo(item);
                $('<input type="date" class="to modified datepicker">').appendTo(item).val($.anura.getState(this.options.state, 'search-modified-before', ''));
            }
            if (widget.size !== false) {
                this.facet_fields.push('fileProperties_int_width');
                this.facet_fields.push('fileProperties_int_height');
                $('<p />').text(this.strings['AssetProperties.width'] + ':').appendTo(item);
                $('<input type="number" class="from width">').appendTo(item);
                $('<span class="range"> - </span>').appendTo(item);
                $('<input type="number" class="to width">').appendTo(item);
                $('<p />').text(this.strings['AssetProperties.height'] + ':').appendTo(item);
                $('<input type="number" class="from height">').appendTo(item);
                $('<span class="range"> - </span>').appendTo(item);
                $('<input type="number" class="to height">').appendTo(item);
            }
            if (this.options.datetimepicker && jQuery().datetimepicker) item.find('.datepicker').datetimepicker(this.options.datetimepicker);
            item.find('input').on('keypress', function (e) {
                if (e.keyCode === 13 && $(this).val() !== '') _this.element.find('.anura-search-button').click();
            });

        },
        createTreeWidget: function(widget, id_ext) {
            this.facet_fields.push('containers');
            var container_class = widget.container_class?widget.container_class:this.options.container_class;
            var field_container = $('<div />').addClass('anura-field-container '+container_class).appendTo(this.container);
            var item = $('<fieldset />').addClass('anura-search-keywords anura-search-tree').appendTo(field_container), _this = this;
            var legend = $('<legend>' + ((widget.title)?(widget.title[_this.options.locale]?widget.title[_this.options.locale]:widget.title):this.strings['general.keywords']) + '<span/></legend>').appendTo(item);
            if (widget.collapsible) { // TODO collapsed value indidator
                legend.on('click', function () {$(this).parent().toggleClass('anura-search-collapsed');});
                item.addClass('anura-search-collapsible ' + (widget.collapsed ? 'anura-search-collapsed anura-search-default-collapsed' : 'anura-search-default-not-collapsed'));
            }
            var list_target = $('<ul />').addClass('anura-search-keyword-holder').appendTo(item);
            var tree_target = $('<div />').addClass('anura-search-keyword-tree').attr('id', 'anura-search-tree-' + widget.root).appendTo(item);
            var state = (this.options.state !== false) ? (this.options.state + 'search-kw' + widget.root + '-') : false;
            var unselector = (widget.unselectable === false) ? -1 : null, submit_on_click = widget.autosubmit;  // TODO migrate unselector feature to anuraTree
            var tree_settings = $.extend({}, {
                server: this.options.server, ext: this.options.ext, locale: this.options.locale, anchor: false, state: state, asset_count: true, hide_empty: true,
                node: {kind: widget.kind ? widget.kind : 103, id: widget.root}, callback: function (server, kind, id, name) {
                    if (id === unselector) { // provide a machanism to unselect a previously selected node
                        tree_target.find('.anura-active .anura-tree-unfolded').addClass('anura-tree-folded').removeClass('anura-tree-unfolded');
                        tree_target.find('li').removeClass('anura-active anura-current');
                        tree_target.data('selected', null);
                        $.anura.removeState(state, 'tree');
                        unselector = null;
                    } else if (unselector !== -1) {
                        unselector = id;
                    }
                    _this.element.trigger('anura-keywords-clicked', [server, kind, id, name]);
                    if (submit_on_click) _this.doSearch();
                }
            }, widget.tree_settings);
            this.rpcStart('keywords');
            // TODO offer dropdown like for noderefs
            tree_target.anuraTree(tree_settings).on('anura-init', function () {
                if (unselector !== -1) unselector = tree_target.data('selected') || null;
                _this.element.trigger('anura-init-keywords', _this.options);
                _this.rpcDone('keywords');
            }).on('anura-reset', function () {
                unselector = (widget.unselectable === false) ? -1 : null;
            });
            if (this.multiple) $('<a class="anura-search-keyword-add">+ '
                + this.strings['lookup.addKeywordButton'] + '</a>').appendTo(item).on('click', function () {
                var selected = tree_target.data('selected');
                if (selected) {
                    var li = $('<li />').data('id', selected).html('+ ' + tree_target.find('#anura-node-' + selected + ' > a').text())
                        .on('click', function () {
                            $(this).remove();
                        }).appendTo(list_target);
                    tree_target.find('li').removeClass('anura-active anura-current'); // reset tree
                    tree_target.find('ul').addClass('anura-tree-folded').removeClass('anura-tree-unfolded');
                    if (submit_on_click) _this.doSearch();
                }
            });
            this.container.on('anura-facet-update', function(event, data) {
                if (data && data.containers) {
                    tree_target.find('.anura-asset-count').text('0').closest('li').addClass('anura-facet-none');
                    $.each(data.containers, function(k, c) {
                        tree_target.find('#anura-node-'+k+' > a .anura-asset-count').text(c).closest('li').removeClass('anura-facet-none');
                    });
                    field_container.toggleClass('anura-facet-none', tree_target.find('.anura-tree-container li').not('.anura-facet-none').length === 0);
                }
            });
        },
        createInfofieldWidget: function(widget, id_ext) {
            this.facet_fields.push('info_'+widget.id);
            var container_class = widget.container_class?widget.container_class:this.options.container_class;
            var field_container = $('<div />').addClass('anura-field-container anura-field-container-info-'+widget.id+' '+container_class).appendTo(this.container);
            var fieldset = $('<fieldset />').addClass('anura-search-infofield id-' + widget.id).data('id', widget.id).appendTo(field_container), _this = this;
            this.rpcStart('info-'+widget.id+'-props');
            $.anura.getJSON(this, this.options.server + '/asset' + this.options.ext + 'callback=?',
                    {infofield: widget.id, locale: this.options.locale}, 'anuraSearchInfo' + widget.id + (widget.root_node ? 'n'+widget.root_node : ''), function (data) {
                if (!data || data.name === undefined) {
                    _this.rpcDone('info-'+widget.id+'-props-nodata');
                    return fieldset.html('failed to retrieve infofield ' + widget.id + '<br>check id and permissions');
                }
                var legend = $('<legend>' + ((widget.title)?(widget.title[_this.options.locale]?widget.title[_this.options.locale]:widget.title):data.name) + '<span/></legend>').appendTo(fieldset);
                if (widget.collapsible) {
                   legend.on('click', function () {$(this).parent().toggleClass('anura-search-collapsed');});
                   fieldset.addClass('anura-search-collapsible ' + (widget.collapsed ? 'anura-search-collapsed anura-search-default-collapsed' : 'anura-search-default-not-collapsed'));
                }
                if (data.description) legend.attr('title', data.description).addClass('anura-search-has-description');
                var additional_class = '', input_type = 'text', infoState = $.anura.getState(_this.options.state, 'search-info-' + widget.id, '');
                if (!infoState && widget.default) infoState = widget.default;
                if ($.isFunction(_this.options.infostate_transform)) infoState = _this.options.infostate_transform(widget, infoState);
                switch (data.type) {
                    case 4: // checkbox
                    case 5: // dropdown
            // cb/dd as dropdowns
                        if (widget.style === 'dropdown' || widget.style === 'select') {
                            var select = $('<select>').addClass(additional_class).appendTo(fieldset);
                            if (widget.style === 'select') select.attr('multiple', true);
                            if (widget.unselectable !== false) $('<option />').addClass('anura-dd-empty').val('').text('').appendTo(select);
                            $.each(data.values, function () {
                                if (data.type === 4) this.id = _this.normalizeBoolean(this.id);
                                if (this.id !== '' || widget.unselectable === false) {
                                    $('<option />').val(this.id).data('id', this.id).text(this.name)
                                        .prop('selected', this.id === infoState).appendTo(select);
                                }
                            });
                            if (widget.autosubmit) select.on('change', function() {_this.doSearch();});
                            if (jQuery().chosen) select.chosen(_this.options.chosen);

                            _this.container.on('anura-facet-update', function(event, data) {
                                var facets = data['info_'+widget.id];
                                if (!$.isEmptyObject(facets)) {
                                    select.find('option').not('[value=""]').prop('disabled', true).addClass('anura-facet-none');
                                    $.each(facets, function(id, count) {
                                        id = _this.normalizeBoolean(id);
                                        _this.setOptionFacetCount(select.find('option[value='+id+']').prop('disabled', false).removeClass('anura-facet-none'), count);
                                    });
                                    field_container.removeClass('anura-facet-none');
                                } else {
                                    field_container.addClass('anura-facet-none');
                                }
                                if (jQuery().chosen) select.trigger('chosen:updated');
                            });
            // cb/dd as cb/radio
                        } else {
                            if (widget.style !== 'checkbox' || data.type === 4) widget.style = 'radio';
                            $.each(data.values, function () {
                                if (data.type === 4) this.id = _this.normalizeBoolean(this.id);
                                var uid = 'anura-info-' + widget.id + '-' + this.id;
                                var typediv = $('<div/>').addClass(uid).appendTo(fieldset);
                                if (widget.style === 'checkbox' && this.id === '') return;
                                $('<input type="' + widget.style + '">').attr('name', 'info-' + widget.id).addClass(additional_class)
                                    .val(this.id).data('id', this.id).attr('id', uid).appendTo(typediv)
                                    .prop('checked', this.id === infoState);
                                var label = $('<label />').attr('for', uid).text(this.name).appendTo(typediv);
                                if (_this.options.facets) $('<span>').addClass('anura-asset-count').appendTo(label);
                            });
                            if (widget.autosubmit) fieldset.find('input').on('change', function() {_this.doSearch();});

                            _this.container.on('anura-facet-update', function(event, data) {
                                var facets = data['info_'+widget.id];
                                if (!$.isEmptyObject(facets)) {
                                    field_container.find('input').not('#anura-info-' + widget.id + '-').prop('disabled', true);
                                    field_container.find('label').not('[for=anura-info-' + widget.id + '-]')
                                        .find('.anura-asset-count').text(0).closest('div').addClass('anura-facet-none');
                                    $.each(facets, function(id, count) {
                                        id = _this.normalizeBoolean(id);
                                        field_container.find('#anura-info-' + widget.id + '-' + id).prop('disabled', false).closest('div').removeClass('anura-facet-none');
                                        field_container.find('label[for="anura-info-' + widget.id + '-' + id +'"] .anura-asset-count').text(count);
                                    });
                                    field_container.removeClass('anura-facet-none');
                                } else {
                                    field_container.addClass('anura-facet-none');
                                }
                            });
                        }
                        break;
                    case 3: // noderef
                        var target = $('<div />').addClass('anura-search-noderef').attr('id', 'anura-search-noderef-' + widget.id)
                            .data('recursive', widget.recursive !== false).data('and-mode', widget.and_mode === true).appendTo(fieldset); // TODO checkbox in UI for this
                        var root_node = widget.root_node?widget.root_node:data.root_node;
            // noderef dropdown/select
                        if (widget.style === 'dropdown' || widget.style === 'select' || widget.style === 'checkbox' || widget.style === 'radio') {
                            _this.rpcStart('info-' + widget.id + '-node-dd');
                            $.anura.getJSON(_this, _this.options.server + '/node' + _this.options.ext + 'callback=?', {
                                    tree: root_node,
                                    locale: _this.options.locale,
                                    noderef: true,
                                    recursive: true,
                                    asset_count: true,
                                    depth: widget.depth ? widget.depth : -1
                                },
                                'anuraTreeDropdown' + widget.id + 'n' + root_node, function (tdata) {
                                    if (widget.style === 'dropdown' || widget.style === 'select') {
                                        var select = $('<select>').addClass(additional_class).appendTo(target);
                                        if (widget.style === 'select') select.prop('multiple', true);
                                        if (widget.style === 'dropdown' && widget.unselectable !== false) {
                                            $('<option />').addClass('anura-dd-empty').val('').text('').appendTo(select);
                                        }
                                    }
                                    var appendToDropdown = function (items, indent, level) {
                                        if (widget.sort !== false) {
                                            if ($.isFunction(widget.sort)) items.sort(widget.sort);
                                            else if (widget.sort === 'desc') items.sort(function (a, b) {
                                                return b.name.localeCompare(a.name, undefined, {numeric: true, sensitivity: 'base'});});
                                            else items.sort(function (a, b) {
                                                return a.name.localeCompare(b.name, undefined, {numeric: true, sensitivity: 'base'});});
                                        }
                                        var infoStates = (infoState+'').split(/[,+]/); // handle multi-values (TODO handle in other widget types)
                                        if (infoStates.length > 1) infoStates.pop(); // pop recursiveness
                                        $.each(items, function () {
                                            if (this.assets > 0) {
                                                if (widget.style === 'dropdown' || widget.style === 'select') {
                                                    var opt = $('<option />').val(this.id).html(indent + this.name).appendTo(select)
                                                        .prop('selected', infoStates.indexOf(this.id.toString()) !== -1 || infoStates.indexOf(this.name) !== -1)
                                                        .data('asset-count', this.assets);
                                                } else {
                                                    var input_id = 'anura-search-noderef-' + widget.id + '-' + this.id;
                                                    var checkdiv = $('<div>').addClass(input_id).appendTo(target);
                                                    var opt = $('<input />').attr('type', widget.style).val(this.id).appendTo(checkdiv).attr('id', input_id)
                                                    .prop('checked', infoStates.indexOf(this.id.toString()) !== -1 || infoStates.indexOf(this.name) !== -1)
                                                    .data('asset-count', this.assets);
                                                    if (widget.style === 'radio') opt.attr('name', 'anura-search-noderef-' + widget.id);
                                                    $('<label />').attr('for', input_id).html(indent + this.name+' <span class="anura-asset-count"></span>').appendTo(checkdiv);
                                                }
                                                if (this.children) {
                                                    appendToDropdown(this.children, indent + (widget.indent ? widget.indent : ''), level + 1);
                                                    opt.addClass('anura-dd-node anura-dd-level-'+level);
                                                } else {
                                                    opt.addClass('anura-dd-leaf anura-dd-level-'+level);
                                                }
                                            }
                                        });
                                    };
                                    appendToDropdown(tdata, '', 0);
                                    if (jQuery().chosen && select) select.chosen(_this.options.chosen);
                                    if (widget.autosubmit) (select || target.find('input')).on('change', function () {
                                        _this.doSearch();
                                    });

                                    _this.container.on('anura-facet-update', function (event, data) {
                                        var facets = data['info_' + widget.id];
                                        /*if (widget.and_mode !== true) {*/ // AND mode allows intersections that yield nothing -> prevent facet-none (otherwise you can't unselect)
                                        target.find('option').not(':selected').not('.anura-dd-empty').prop('disabled', true).addClass('anura-facet-none');
                                        target.find('input').not(':checked, .chosen-search-input').prop('disabled', true).parent().addClass('anura-facet-none');
                                        /*}*/
                                        if (!$.isEmptyObject(facets)) {
                                            $.each(facets, function (id, count) {
                                                _this.setOptionFacetCount(target.find('option[value="' + id + '"]')
                                                    .prop('disabled', false).removeClass('anura-facet-none'), count);
                                                target.find('input[value="' + id + '"]').prop('disabled', false)
                                                    .parent().removeClass('anura-facet-none').find('.anura-asset-count').text(count);
                                            });
                                            field_container.removeClass('anura-facet-none');
                                        } else {
                                            if (widget.and_mode !== true || (select && !select.val())) field_container.addClass('anura-facet-none');
                                        }
                                        if (jQuery().chosen) fieldset.find('select').trigger('chosen:updated');
                                    });
                                    _this.element.trigger('anura-init-rpc-' + widget.id, this.options);
                                    _this.rpcDone('info-' + widget.id + '-node-dd');
                                });
            // noderef by text
                        } else if (widget.style === 'text') {
                            var text = $('<input type="text">').val(infoState?infoState.replace(/"/g, '').replace(/\+/g, ' '):'')
                                    .addClass('text '+additional_class).appendTo(target);
                            if (widget.autosubmit) text.on('change', function () {
                                _this.doSearch();
                            });
            // noderef tree
                        } else {
                            var unselector = (widget.unselectable === false) ? -1 : null; // TODO migrate unselector feature to anuraTree
                            var state = (_this.options.state !== false) ? (_this.options.state + 'search-nr' + widget.id + '-') : false;
                            var tree_settings = $.extend({}, {server: _this.options.server, ext: _this.options.ext, locale: _this.options.locale, depth: widget.depth ? widget.depth : -1,
                                node: {id: root_node}, anchor: false, state: state, show: false, noderef: true, recursive: true, asset_count: true, hide_empty: true},
                                widget.tree_settings /*TODO provide widget.multiple to tree's multiselect */, { callback: function (server, kind, id, name) {
                                    if (id === unselector) { // provide a machanism to unselect a previously selected node
                                        target.find('.anura-active .anura-tree-unfolded').addClass('anura-tree-folded').removeClass('anura-tree-unfolded');
                                        target.find('li').removeClass('anura-active anura-current');
                                        target.data('selected', null);
                                        $.anura.removeState(state, 'tree');
                                        unselector = null;
                                    } else if (unselector !== -1) {
                                        unselector = id;
                                    }
                                    _this.element.trigger('anura-keywords-clicked', [server, kind, id, name]);
                                    if (widget.autosubmit) _this.doSearch();
                                }
                            });
                            _this.rpcStart('info-'+widget.id+'-node-tree');
                            target.anuraTree(tree_settings).on('anura-init', function () {
                                if (unselector !== -1) unselector = target.data('selected') || null;
                                _this.element.trigger('anura-init-rpc-' + widget.id, _this.options);
                                _this.rpcDone('info-'+widget.id+'-node-tree');
                            }).on('anura-clicked', function (e, server, kind, id, name) {
                                legend.find('span').html((target.find('.anura-current').length) ? (': <span class="anura-name">' + name + '</span>') : '');
                            }).on('anura-reset', function () {
                                unselector = (widget.unselectable === false) ? -1 : null;
                            });

                            _this.container.on('anura-facet-update', function(event, data) {
                                var facets = data['info_'+widget.id];
                                fieldset.find('li').addClass('anura-facet-none');
                                fieldset.find('.anura-asset-count').text('0');
                                if (!$.isEmptyObject(facets)) {
                                    $.each(facets, function(key, count) {
                                        fieldset.find('.anura-search-noderef #anura-node-'+key).removeClass('anura-facet-none').find('> a .anura-asset-count').text(count);
                                    });
                                    field_container.removeClass('anura-facet-none');
                                } else {
                                    field_container.addClass('anura-facet-none');
                                }
                            });
                        }
                        if (widget.recursive_toggle) {
                            $('<input type="checkbox">').addClass('anura-search-noderef-recursive-toggle')
                                .prop('checked', widget.recursive).appendTo(target.addClass('anura-search-noderef-has-recursive-toggle'))
                                .on('change', function () {
                                    target.data('recursive', $(this).is(':checked'));
                                    if (widget.autosubmit) _this.doSearch();
                                });
                        }
                        break;
                    default:
                        if (data.type === 1) {input_type = 'number';}
                        if (data.type === 2) {
                            input_type = widget.type === 'month' ? 'month' : 'date'; /* month does not work in FF -.- */
                            additional_class = 'datepicker';
                        }
                        if (widget.style === 'range') {
                            var infoStates = infoState.split(',');
                            $('<input type="'+input_type+'" class="from ' + additional_class + '">').attr('placeholder', widget.placeholder)
                                    .appendTo(fieldset).val(infoStates[0]);
                            $('<span class="range"> - </span>').appendTo(fieldset);
                            $('<input type="'+input_type+'" class="to ' + additional_class + '">').attr('placeholder', widget.placeholder)
                                    .appendTo(fieldset).val(infoStates.length > 1 ? infoStates[1] : '');
                        } else {
                            var input = $('<input type="'+input_type+'" class="text ' + additional_class + '">').attr('placeholder', widget.placeholder)
                                    .appendTo(fieldset).val(infoState.replace(/\+/g, ' '));
                            if (_this.options.facets && input_type === 'text'  && widget.datalist !== false) {
                                input.attr('list', 'anura-infodl-'+widget.id);
                                var datalist = $('<datalist/>').attr('id', 'anura-infodl-'+widget.id).appendTo(fieldset);
                                _this.container.on('anura-facet-update', function(event, data) {
                                    var facets = data['info_'+widget.id];
                                    if (!$.isEmptyObject(facets)) {
                                        datalist.find('option').remove();
                                        $.each(facets, function (name, count) {
                                            if (name) { // TODO prettyName conversion leads to dupes -> dedupe?
                                                var prettyName = (name.charAt(0).toUpperCase() + name.slice(1)).replace(/\W/, '');
                                                if (prettyName.length > 3) {
                                                    $('<option/>').attr('value', prettyName).text(prettyName + " [" + count + "]").appendTo(datalist);
                                                }
                                            }
                                        });
                                        field_container.removeClass('anura-facet-none');
                                    } else {
                                        field_container.addClass('anura-facet-none');
                                    }
                                });
                            }
                        }
                        if (data.min) fieldset.find('input').attr('min', data.min);
                        if (data.max) fieldset.find('input').attr('max', data.max);
                        break;
                }
                if (_this.options.datetimepicker && jQuery().datetimepicker) fieldset.find('.datepicker').datetimepicker(_this.options.datetimepicker);
                fieldset.find('input').on('keypress', function (e) {
                    if (e.keyCode === 13 && $(this).val() !== '') _this.doSearch();
                });
                if (widget.autosubmit) fieldset.find('input').on('change', function() {_this.doSearch();});
                _this.element.trigger('anura-init-' + widget.id, _this.options);
                _this.rpcDone('info-'+widget.id+'-props');
            }, function(error_response) {
                if (console) console.log('error '+error_response.status+' while retrieving infofield '+widget.id+': '+error_response.message);
                _this.rpcDone('info-'+widget.id+'props-fail');
            });
        },
        setOptionFacetCount: function(option, count) {
            if (!option || !option.length || !this.options.facet_count) return;
            var title = option.text(), exInd = title.lastIndexOf(' [');
            option.text((exInd !== -1 ? title.substr(0, exInd) : title) + ' ['+count+']').data('asset-count', count);
        },
        resizeChosen: function () {
            if (jQuery().chosen && this.container) this.container.find('.chosen-container').each(function () {
                $(this).attr('style', 'width: 100%');
            });
        },
        normalizeBoolean: function(value) {
            switch (value) {
                case 'true': return 1;
                case 'false': return 0;
                case '0': return this.version > 2 && this.version < 2.7 ? 1 : 0;
                case '1': return this.version > 2 && this.version < 2.7 ? 0 : 1;
                default: return value;
            }
        }
    };
    $.fn[pluginName] = function ( options ) {
        return this.each(function (i, item) { // iterate matches
            var instance = $.data(item, "plugin_" + pluginName);
            if (!instance) { // if it has no instance yet, create it
                $.data(item, "plugin_" + pluginName, new Plugin( item, options ));
            } else { // otherwise merge new options with existing ones, empty and re-init
                instance.options = $.extend( {}, instance.options, options );
                instance.init();
            }
        });
    };
})( jQuery, window, document );
