/* © brix cross media AG, http://www.brix.ch - It is strictly forbidden to change, reuse or copy the source code of this document. */
;(function ( $, window, document, undefined ) {
    var pluginName = "anuraTable",
        defaults = {
            server: 'https://celum.local/anura/first', // REQUIRED, which anura endpoint to use
            ext: '.do?', // extension of server pages (.php, .do)
            node: {kind: 101, id: 0}, // which node of what kind to show (101 = folder, 102 = collection, 103 = keyword)
            top: {number: 0, days: 0}, // show the top X downloads within the last Y days
            relations: {id: 0, version: null, type: null}, // show related assets of a particular asset
            assets: null, // load a defined list of assets from somewhere else
            search: {string: '', node: 0, keyword: 0},
            columns: ['icon', 'name', 'size', 'date', 'download'], // any of: name, thumb, icon, size, extension, date, time, download, pages, alt_name or any other property returned by the API. Infofields with info.x, e.g. info.101
            headings: {}, // headings that should be used for non-supported columns ({key: value, ...})
            sort: null, // order: one of [id, name, uploaddate, lastmodified, extension, assettype], seq: asc or desc - i.e. {order: 'name', seq: 'asc'}
            sortable: true, // enable sorting columns by clicking on the table headers
            locale: '', // ISO 639-1 code, blank for default
            icons: true, // DEPRECATED (use columns), true to show file type icons
            paginate: 16, // how many assets to fetch at once when paginage is set to 'auto'
            paginator: ['prev', 'next', 'pages', 'counter'],
            paginator_nav: {prev: '&larr;', next: '&rarr;'},
            autoload_fetch: 50, // how many assets to fetch at once when paginage is set to 'auto'
            limit: 0, // limit the results (or reported total) returned by the server
            detail_view: 'anura', // one of: slimbox, colorbox, details, blank, none
            detail_settings: {}, // settings to pass to anura.details.js
            colorbox_min_height: 500, colorbox_settings: {}, // colorbox specific settings since detail_settings is used by anuraDetails
            alternative_name: { // alternative name (for slimbox etc.)
                id: 0, // information field ID
                always: false, //always use the alternative name, not just for slimbox
                heading: '' // title when used in column
            },
            download_directly: true, // make the different formats available directly
            icon_transform: function(icon) {return icon.replace('/big/', '/small/')},
            hover_thumbnail: '.anura-thumb, .anura-icon, .anura-name', // selector for showing a thumbnail on hover, or false to disable
            force_zip: false, // force even single asset downloads to be delivered in a ZIP-file
            basket: null, // object to use as a basket, initialized by a basket plugin
            state: '', // prefix for state in url (multi-instance) or false to disable state
            send: false, // add a mailto-link (ID of the download format to use or true for the first)
            stream: false, // enables audio and video streaming - use with caution, might overload the server
            autoplay: true, // whether to automatically start playing (with vimeo and/or stream)
            recursive: false, // load assets from all subnodes recursively (like keywords in 1.x)
            noderef: false, // show assets that have node referencing infofields to this node, 2.x
            detail_trigger: true, // listen to the details=... parameter and show the assets details view (regardless of actual navigation context)
            use_create_date: false, // prefer the creation date over the modified date in the date/time columns
            date_format: 'locale', // format tho show dates in, currently supports either 'locale' or 'iso' or {time: 'HH:MM', date: 'dd.mm.yyyy', both: 'dd.mm.yyyy HH:MM'}
            text_length_limit: false, // limit the length of the asset name and infofields to a certain number of characters
            lightbox_rel: 'lightbox-anura', // name of the rel attribute to use for lighbox triggers
            row_trigger: false // when any cell in a row is clicked, trigger the lightbox to open (vla a[rel=lightbox_rel])
        };
    function Plugin( element, options ) {
        this.element = this.container = $(element);
        if (!options.lightbox_rel && options.state) options.lightbox_rel = 'lightbox-anura-'+options.state;
        this.options = $.extend( {}, defaults, options );
        this._defaults = defaults;
        this._name = pluginName;
        this.download_formats = null;
        this.strings = {};
        this.strings_locale = this.options.locale;
        this.infofields = [];
        // legacy syntax
        if (this.options.icons && $.inArray('icon', this.options.columns) === -1) this.options.columns.unshift('icon');
        if (this.options.hover_thumbnail === true) this.options.hover_thumbnail = this._defaults.hover_thumbnail;
        this.init();
    }
    Plugin.prototype = {
        init: function() {
            var _this = this;
            if (!$.isEmptyObject(this.strings) && this.options.locale === this.strings_locale) {
                this.loadTable();
            } else {
                var stringsToLoad = ['AssetProperties.fileExtension', 'DocDocumentProperties.pages', 'MediaAssetProperties.filesize', 'anura.addToBasket', 'anura.loadMore',
                    'PdfDocumentProperties.modifiedDate', 'search.modified', 'MENUITEMID_DOWNLOAD', 'object.originalFileName', 'TgaImageProperties.creationDate'];
                $.each(this.options.columns, function(index, column) {
                    if (column.indexOf('info.') === 0) {
                        stringsToLoad.push('anura.'+column);
                        _this.infofields.push(column.replace('info.', ''));
                    }
                });
                $.anura.getJSON(this, this.options.server+'/asset'+this.options.ext+'callback=?&string='+stringsToLoad.join('&string='),
                    {locale: this.options.locale}, 'anuraTableStrings', function(data) {
                        _this.strings = data;
                        _this.strings_locale = _this.options.locale;

                        // transitionally fetch global DLFs -> to be replaced by individual calls
                        if (_this.options.download_directly && _this.download_formats == null) {
                            $.anura.getJSON(_this, _this.options.server+'/asset'+_this.options.ext+'callback=?',
                                {downloadformats: 1, locale: _this.options.locale}, 'anuraTableDlf', function(dlfs) {
                                    _this.download_formats = dlfs;
                                    if (_this.options.send === true) _this.options.send = dlfs[0].id;
                                    _this.loadTable();
                                });
                        } else {
                            _this.loadTable();
                        }
                    });
            }
        },
        loadTable: function() {
            var previous = this.element.find('.anura-table-container'), _this = this;
            this.container = ((previous.length > 0)?previous.first():$('<div />').addClass('anura-table-container').appendTo(this.element))
                .removeClass('nothing').removeClass(function (index, css) {return (css.match (/anura-locale-\S+/g) || []).join(' ');})
                .addClass('anura-loading anura-locale-'+this.options.locale);
            this.element.trigger('anura-loading', [this.options]);
            this.createTable();

            this.pageCall = {url: this.options.server+'/node'+this.options.ext+'callback=?', name: 'anuraTable',
                params: {locale: this.options.locale, alt_name: this.options.alternative_name.id, extended: true}};
            if (this.options.sort) {
                if (typeof this.options.sort === "string") {
                    var s = this.options.sort.split('_'); this.options.sort = {};
                    this.options.sort.seq = s.pop();
                    this.options.sort.order = s.join('_'); // join again for info_123
                }
                var sort_direction = this.options.sort.seq || 'asc';
                $.extend(this.pageCall.params, {sort_order: this.options.sort.order, sort_order_seq: sort_direction});
                this.container.find('th.anura-heading-'+this.options.sort.order).addClass(sort_direction);
            }
            if (this.infofields.length > 0) $.extend(this.pageCall.params, {infofields: this.infofields.join(',')});
            if (this.options.limit > 0) this.pageCall.params.limit = this.options.limit;

            if (this.options.assets) {
                return this.populateContainer(this.options.assets);
            } else if (this.options.search && (this.options.search.custom || this.options.search.string)) {
                this.pageCall.name = 'anuraTableSearch';
                this.pageCall.url = this.options.server+'/node'+this.options.ext+'callback=?'+((this.options.search.custom)?('&'+this.options.search.custom):'');
                $.extend(this.pageCall.params, {search: this.options.search.string, search_node: this.options.search.node});
                this.options.search.string = '';
            } else if (this.options.node && this.options.node.id) {
                this.pageCall.name = 'anuraTableNode';
                $.extend(this.pageCall.params, {kind: this.options.node.kind, id: this.options.node.id,
                    recursive: this.options.recursive, noderef: this.options.noderef});
            } else if (this.options.top && this.options.top.number) {
                this.pageCall.name = 'anuraTableTopDl';
                $.extend(this.pageCall.params, {top: this.options.top.number, days: this.options.top.days});
            } else if (this.options.relations && this.options.relations.id) {
                this.pageCall.name = 'anuraTableRel'+this.options.relations.id;
                $.extend(this.pageCall.params, {relations: this.options.relations.id});
                if (this.options.relations.version) $.extend(this.pageCall.params, {version: this.options.relations.version});
                if (this.options.relations.type) $.extend(this.pageCall.params, {type: this.options.relations.type});
            } else {
                this.container.removeClass('anura-loading');
                return;
            }
            this.element.data('page-call', this.pageCall);

            this.showPage(0);

            if (this.options.detail_trigger) { // TODO have utils do this
                var detailAssetId = $.anura.getState(this.options.state, 'details');
                if (detailAssetId) {
                    var lightboxSelector = ' a[rel*='+this.options.lightbox_rel+']'+(_this.options.detail_view === 'swipebox'?' img':'');
                    this.element.on('anura-loaded-first.detail-trigger', function() {
                        _this.element.off('anura-loaded-first.detail-trigger');
                        _this.options.detail_trigger = false; // only on first run
                        if (_this.element.find('#anura-asset-id'+detailAssetId).length === 0) {
                            $.anura.getJSON(_this, _this.pageCall.url, {search_id: detailAssetId}, 'anuraAsset'+detailAssetId, function (data, response) {
                                if (data[0]) {
                                    _this.populateContainer(data);
                                    _this.element.find('#anura-asset-id'+detailAssetId).hide().find(lightboxSelector).click();
                                }
                            });
                        } else if (_this.options.detail_view === 'anura') {
                            _this.element.anuraLightbox({action: detailAssetId});
                        } else {
                            _this.element.find('#anura-asset-id'+detailAssetId+lightboxSelector).click();
                        }
                    });
                }
            }
        },
        createTable: function() {
            var _this = this;
            var table = $('<table />').appendTo($('<div>').addClass('anura-table-wrapper').appendTo(this.container.empty()));
            var colgr = $('<colgroup />').appendTo(table), thead = $('<thead />').appendTo(table),
                tbody = $('<tbody />').appendTo(table), row = $('<tr />').appendTo(thead);
            $.each(this.options.columns, function(index, column) {
                var isInfofield = column.indexOf("info.") === 0;
                $('<col />').addClass('anura-column '+(isInfofield?'anura-column-info ':'')+'anura-column-'+column.replace(/[._]/g, '-')).appendTo(colgr);
                switch (column) {
                    case 'name':
                        var th = $('<th />').addClass('anura-heading-name').data('sort-key', 'name')
                            .text(_this.strings['object.originalFileName']).appendTo(row);
                        var alt_n = _this.options.alternative_name;
                        if (!alt_n || alt_n.id === 0) th.addClass('anura-sortable');
                        else if (alt_n.sortable !== false) th.addClass('anura-sortable').data('sort-key', 'info_'+alt_n.id);
                        if (alt_n.heading) th.text(alt_n.heading[_this.options.locale]?alt_n.heading[_this.options.locale]:alt_n.heading);
                        break;
                    case 'id':
                        var th = $('<th />').addClass('anura-heading-id anura-sortable').data('sort-key', 'id').text('ID').appendTo(row);
                        break;
                    case 'extension':
                        var th = $('<th />').addClass('anura-heading-extension anura-sortable').data('sort-key', 'extension')
                            .text(_this.strings['AssetProperties.fileExtension']).appendTo(row);
                        break;
                    case 'pages':
                        $('<th />').addClass('anura-heading-pages').text(_this.strings['DocDocumentProperties.pages']).appendTo(row);
                        break;
                    case 'size':
                        var th = $('<th />').addClass('anura-heading-size').text(_this.strings['MediaAssetProperties.filesize']).appendTo(row);
                        break;
                    case 'date':
                    case 'time':
                    case 'date_time':
                    case 'created_date':
                    case 'created_time':
                    case 'created_date_time':
                    case 'modified_date':
                    case 'modified_time':
                    case 'modified_date_time':
                        var createdMode = column.indexOf('created') === 0;
                        if (!createdMode && column.indexOf('modified') === -1) createdMode = _this.options.use_create_date; // legacy
                        var th = $('<th />').addClass('anura-heading-'+column.replaceAll(/_/g, '-')+' anura-sortable')
                            .data('sort-key', createdMode ? 'uploaddate' : 'lastmodified')
                            .text(_this.strings[createdMode ? 'TgaImageProperties.creationDate' : 'PdfDocumentProperties.modifiedDate']).appendTo(row);
                        break;
                    case 'alt_name':
                        var th = $('<th />').addClass('anura-heading-alt-name').text(_this.options.alternative_name.heading).appendTo(row);
                        break;
                    case 'download':
                        var th = $('<th />').addClass('anura-heading-download').text(_this.strings['MENUITEMID_DOWNLOAD']).appendTo(row);
                        break;
                    default:
                        if (isInfofield) {
                            var th = $('<th />').addClass('anura-heading-info anura-sortable anura-heading-'+column.replace(/[._]/g, '-'))
                                .text(_this.strings['anura.'+column]).data('sort-key', column.replace(/[._]/g, '_')).appendTo(row);
                        } else {
                            var th = $('<th />').addClass('anura-heading-'+column).text(_this.options.headings[column]).appendTo(row);
                        }
                        break;
                }
                if (th && th.text()) th.attr('title', th.text());
            });
            if (this.options.sortable) thead.find('th.anura-sortable').on('click', function() {
                var sort_direction = ($(this).hasClass('asc'))?'desc':'asc';
                thead.find('th').removeClass('asc desc');
                $(this).addClass(sort_direction);
                _this.options.sort = {order: $(this).data('sort-key'), seq: sort_direction};
                $.extend(_this.pageCall.params, {sort_order: $(this).data('sort-key'), sort_order_seq: sort_direction});
                _this.showPage(0);
            });
        },
        showPage: function(page) {
            var _this = this;
            this.container.addClass('anura-loading');
            if (this.options.paginate === 'auto' || this.options.paginate === 'button') {
                this.container.find('.anura-autoload').remove();
                var autoload = $('<div/>').addClass('anura-autoload').appendTo(this.container), autoload_page = page;
                var doInfiniScroll = function() {
                    var button = autoload.find('button');
                    if (button.prop('disabled') === true) return;
                    button.prop('disabled', true);
                    autoload_page = autoload_page + 1;
                    $.extend(_this.pageCall.params, {paginate: _this.options.autoload_fetch, page: autoload_page});
                    $.anura.getJSON(_this, _this.pageCall.url, _this.pageCall.params, _this.pageCall.name, function (data, response) {
                        if (autoload_page === 1) _this.container.find('.anura-asset-container').remove();
                        var total = response && response.total ? response.total : data.length;
                        _this.populateContainer(data, _this.options.autoload_fetch);
                        button.prop('disabled', false);
                        if (_this.container.find('.anura-asset-container:visible').length >= total) autoload.remove();
                        if (autoload_page === 1) _this.element.trigger('anura-loaded-first', [_this.options, total]);
                        if (autoload.is(':appeared')) doInfiniScroll(); // do it again if it's still visible
                    }, function () {
                        autoload.remove();
                    });
                };
                if (this.options.paginate === 'auto') autoload.appear().on('appear', doInfiniScroll);
                $('<button/>').text(this.strings['anura.loadMore'] || 'load more').appendTo(autoload).on('click', doInfiniScroll);
                doInfiniScroll();
            } else {
                $.extend(this.pageCall.params, {paginate: this.options.paginate, page: page+1});
                $.anura.getJSON(this, this.pageCall.url, this.pageCall.params, this.pageCall.name, function(data, response) {
                    _this.container.find('.anura-asset-container').remove();
                    var total = response && response.total ? response.total : data.length;
                    if (!response && total > _this.options.paginate) data = data.slice((page*_this.options.paginate, (page+1)*_this.options.paginate)); // v1 server
                    _this.populateContainer(data, _this.options.paginate);
                    if (_this.options.paginate && $.isNumeric(_this.options.paginate)) {
                        $.anura.addPaginator(_this, page, response?response.total:data.length, _this.container.find('.anura-table-wrapper'));
                    }
                    if (page === 0) _this.element.trigger('anura-loaded-first', [_this.options, total]);
                });
            }
        },
        populateContainer: function(assets, limit) {
            var _this = this, tbody = this.container.find('tbody');
            if (!assets || assets.length < 1) {
                this.container.addClass('empty');
            } else {
                this.container.removeClass('empty');
                if (limit && $.isNumeric(limit)) assets = assets.slice(0, limit);
            }

            $.each(assets, function (index, item) {
                _this.appendAsset(item, tbody, index%2 === 0);
            });
            this.addLightbox();
            if (this.options.hover_thumbnail) {
                this.container.find('#anura-img-hover').remove();
                var hoverer = $('<div>').attr('id', 'anura-img-hover').appendTo(this.container);
                var hover_img = $('<img>').on('load', function () {hoverer.css('max-width', hover_img.width());}).appendTo(hoverer);
                var hover_name = $('<p>').appendTo(hoverer);
                this.container.find(this.options.hover_thumbnail).off('mouseenter').on('mouseenter', function () {
                    var asset = $(this).closest('.anura-asset-container');
                    hover_img.attr('src', _this.options.server + '/asset' + _this.options.ext + 'thumb=' + asset.data('asset-id') +($.anura.tokenProvider?'&token='+$.anura.tokenProvider():'')+ '&img=a.jpg');
                    hover_name.text(asset.data('asset-name'));
                    hoverer.addClass('anura-hovering');
                }).on('mouseleave', function () {
                    hoverer.removeClass('anura-hovering');
                }).on('mousemove', function (e) {
                    var offset = 12, top = e.clientY + offset, left = e.clientX + offset, height = hoverer.height();
                    if (top + height+ offset >= $(window).height()) top -= height + (2*offset);
                    hoverer.css('top',  top+'px').css('left', left+'px');
                });
            }
            this.container.removeClass('anura-loading');
            this.element.trigger('anura-loaded', [this.options, assets]);
        },
        addLightbox: function() {  // TODO have utils do this
            var _this = this;
            switch(this.options.detail_view) {
                case 'slimbox':
                    this.container.find(".anura-asset-container a[rel^='lightbox']").slimbox(this.options.detail_settings, null, function(el) {
                        return (this == el) || ((this.rel.length > 8) && (this.rel === el.rel));
                    });
                    break;
                case 'swipebox':
                    if (baguetteBox !== undefined) baguetteBox.run('.anura-table-container', this.options.detail_settings);
                    else this.container.find(".anura-asset-container a[rel^='lightbox']").swipebox(this.options.detail_settings);
                    break;
                case 'spotlight':
                    $.each(_this.options.detail_settings, function(key, value) {_this.container.attr('data-'+key, value)});
                    _this.container.addClass('spotlight-group'); // setting .data(_this.options.detail_settings) doesn't work?
                    _this.container.find(".anura-asset-container a[rel^='lightbox']").addClass('spotlight');
                    break;
                case 'anura':
                    _this.element.anuraLightbox($.extend({}, _this.options.detail_settings, {
                        server: _this.options.server,
                        locale: _this.options.locale,
                        basket: _this.options.basket,
                        trigger: 'a[rel^='+_this.options.lightbox_rel+']',
                        state: _this.options.state,
                        pageCall: _this.pageCall
                    }));
                    break;
                case 'details':
                    $('#colorbox').addClass('hasAnuraDetails');
                case 'colorbox':
                    var infopanel = function(a) {
                        if (_this.options.detail_view !== 'details') return;
                        var panel = $('#anuraInfoPanel');
                        if (!panel.length) {
                            panel = $('<div id="anuraInfoPanel"/>').appendTo('#cboxContent');
                            $('<button type="button" id="cboxInfo"/>').text('info').on('click', function() {
                                $('#anuraInfoPanel').toggle();
                            }).insertBefore($('#cboxClose'));
                        }
                        if (_this.options.colorbox_min_height) var minHeight = Math.min($(window).height() - 50, _this.options.colorbox_min_height); // workaround for missing minHeight option
                        if ($('#cboxLoadedContent').height() < minHeight) $.colorbox.resize({innerHeight: minHeight+'px'});
                        panel.height($('#cboxLoadedContent').height());
                        if ($(window).width() > 767) panel.show(); //argh
                        var asset_container = $.colorbox.element().closest('.anura-asset-container');
                        var options = $.extend({send: _this.options.send, download_directly: _this.options.download_directly}, _this.options.detail_settings,
                            {asset: asset_container.data('assetId') || asset_container.attr('id').replace(/\D/g,''), alternative_name: _this.options.alternative_name.id,
                                server: _this.options.server, locale: _this.options.locale, basket: _this.options.basket});
                        panel.anuraDetails(options);
                    };
                    var colorbox_options = $.extend({transition: 'none', current: '{current} / {total}'}, _this.options.colorbox_settings,
                        {rel: this.options.lightbox_rel, onComplete: infopanel,
                            onCleanup: function() {$('#anuraInfoPanel').fadeOut()},
                            onClosed: function() {$.anura.removeState(_this.options.state, 'details');}
                        });
                    this.container.find('.anura-asset-container a.image').colorbox($.extend(colorbox_options, {maxWidth: '98%', maxHeight: '98%'}));
                    this.container.find('.anura-asset-container a.vimeo, .anura-asset-container a.youtube').colorbox($.extend(colorbox_options, {iframe:true, innerWidth:960, innerHeight:540}));
                    this.container.find('.anura-asset-container a.soundcloud').colorbox($.extend(colorbox_options, {iframe:true, innerWidth:500, innerHeight:500}));
                    this.container.find('.anura-asset-container a.player').colorbox($.extend(colorbox_options, {iframe:true, innerWidth:640, innerHeight:480}));
                    break;
            }
        },
        appendAsset: function(item, table, even) {
            var _this = this, name = this.options.alternative_name.always?item.alt_name:item.name;
            var row = $('<tr />').attr('id', 'anura-asset-id'+item.id).data('asset-id', item.id).data('asset-name', name).appendTo(table)
                .addClass('anura-asset-container anura-asset-type-'+item.type+' '+(even?'anura-even':'anura-odd'));
            // TODO move this to utils
            if (item.infofields) $.each(item.infofields, function(infoId, infoVal) {row.data('asset-infofield-'+infoId, infoVal);});
            if (item.infofields_raw) $.each(item.infofields_raw, function(infoId, infoVal) {row.data('asset-infofield-raw-'+infoId, infoVal);});
            if (item.downloadable === false) row.addClass('anura-not-downloadable');
            if (item.version) row.data('asset-version', item.version).data('asset-versioned', item.versioned);
            if (item.extension) row.data('asset-extension', item.extension);
            if (item.parents) row.data('asset-parents', item.parents);
            if (item.custom) $.each(item.custom.replace(/_/g, '-').split(' '), function(index, value) {row.addClass('anura-custom-'+value);});
            if (_this.options.extended) row.data('asset-extended', item);
            if (this.options.row_trigger) {
                row.on('click.anura-row-trigger', function (e) {
                    if (!$(e.target).is('a')) row.find('a[rel*=' + _this.options.lightbox_rel + ']').click();
                });
            }

            var token = $.anura.tokenProvider?'&token='+$.anura.tokenProvider():'';
            // add info columns in order
            $.each(this.options.columns, function(index, column) {
                switch (column) {
                    case 'thumb':
                        var imgHolder = $('<div />').addClass('anura-image-container').appendTo($('<td />').addClass('anura-thumb').appendTo(row));
                        $('<img />').data('id', item.id).addClass('anura-image') /* IE */.removeAttr('width').removeAttr('height')/* /IE */
                            .attr('src', _this.options.server+'/asset'+_this.options.ext+'thumb='+item.id+token+'&img=a.jpg').appendTo(imgHolder);
                        imgHolder.on('click', function() {$(this).closest('tr').find('.anura-name a, .anura-name').click();}); // proxy lightbox
                        break;
                    case 'icon':
                        $('<td />').addClass('anura-icon anura-file anura-file-'+item.extension).appendTo(row.data('extension', item.extension));
                        break;
                    case 'name':
                        var nameHolder = $('<td />').addClass('anura-name').appendTo(row.data('name', name));
                        if (_this.options.detail_view !== 'none') {
                            var trigger = $('<a />').attr('rel', _this.options.lightbox_rel+' nofollow').attr('target', '_blank')
                                .html(_this.shortenText(name)).attr('title', name).data('asset-id', item.id).data('asset-extension', item.extension).appendTo(nameHolder);
                            if (!_this.options.hover_thumbnail) trigger.attr('title', name);
                            if ($.isFunction(_this.options.detail_view)) {
                                nameHolder.on('click', function() {_this.options.detail_view(item, trigger)});
                            } else if (_this.options.detail_view === 'download') {
                                trigger.attr('href', _this.options.server+'/asset'+_this.options.ext+'download='+item.id+'&format='+_this.options.detail_settings.format+token);
                            } else if (item.streaming && item.streaming.soundcloud) {
                                trigger.addClass('soundcloud').attr('href', window.location.protocol+'//w.soundcloud.com/player/?url=http://api.soundcloud.com/tracks/'+item.streaming.soundcloud+'?secret_token='+item.streaming.soundcloud_key+'&visual=true'+(_this.options.autoplay?'&auto_play=true':'')+'&buying=false&download=false&show_artwork=true&show_comments=false&show_playcount=false&show_user=false&hide_related=true');
                            } else if (item.streaming && item.streaming.vimeo) {
                                trigger.addClass('vimeo').attr('href', window.location.protocol+'//player.vimeo.com/video/'+item.streaming.vimeo+(_this.options.autoplay?'?autoplay=1':''));
                            } else if (item.streaming && item.streaming.youtube) {
                                trigger.addClass('youtube').attr('href', window.location.protocol+'//www.youtube.com/embed/'+item.streaming.youtube+(_this.options.autoplay?'?autoplay=1':''));
                            } else if (_this.options.stream && (item.type === 3 || item.type === 4)) {
                                trigger.addClass('player').attr('href', _this.options.server+'/asset'+_this.options.ext+'play='+item.id+(_this.options.autoplay?'&autoplay=true':'')+token);
                            } else {
                                trigger.addClass('image').attr('href', _this.options.server+'/asset'+_this.options.ext+'preview='+item.id+token+'&img=a.jpg');
                            }
                        }
                        break;
                    case 'id':
                        $('<td />').addClass('anura-id').text(item.id).appendTo(row.data('id', item.id));
                        break;
                    case 'extension':
                        $('<td />').addClass('anura-extension').text(item.extension).appendTo(row.data('extension', item.extension));
                        break;
                    case 'pages':
                        $('<td />').addClass('anura-pages').text((item.type === 2)?item.pages:'-').appendTo(row.data('pages', item.pages));
                        break;
                    case 'size':
                        $('<td />').addClass('anura-filesize').html(_this.bytesToSize(item.size))
                            .attr('title', item.size+' B').appendTo(row.data('size', item.size));
                        break;
                    case 'date':
                    case 'time':
                    case 'date_time':
                    case 'created_date':
                    case 'created_time':
                    case 'created_date_time':
                    case 'modified_date':
                    case 'modified_time':
                    case 'modified_date_time':
                        var createdMode = column.indexOf('created') === 0;
                        if (!createdMode && column.indexOf('modified') === -1) createdMode = _this.options.use_create_date; // legacy
                        var date = createdMode ? new Date(item.created) : new Date(item.modified);
                        $('<td />').addClass('anura-'+column.replaceAll(/_/g, '-'))
                            .html(_this.dateToString(date, column.replace('created_', '').replace('modified_', '')))
                            .attr('title', _this.dateToString(date).replaceAll(/&nbsp;/g, ' ')).appendTo(row.data('date', date));
                        break;
                    case 'alt_name':
                        $('<td />').addClass('anura-alt-name').html(_this.shortenText(item.alt_name))
                            .attr('title', item.alt_name).appendTo(row.data('alt-name', item.alt_name));
                        break;
                    case 'download':
                        _this.addMenu(row, item);
                        break;
                    default:
                        if (column.indexOf("info.") === 0) {
                            var infoIndex = column.replace('info.', '');
                            var text = (item['infofields'] && item['infofields'][infoIndex])?(item['infofields'][infoIndex]):'';
                            $('<td />').addClass('anura-info anura-'+column.replace('.', '-'))
                                .html(_this.shortenText(text)).attr('title', text).appendTo(row.data('info-'+infoIndex, text));
                        } else {
                            $('<td />').addClass('anura-'+column).html(_this.shortenText(item[column]))
                                .attr('title', item[column]).appendTo(row.data(column, item[column]));
                        }
                        break;
                }
            });
        },
        addMenu: function(target, asset) {
            var _this = this;
            var menu = $('<td />').addClass('anura-asset-menu').appendTo(target);
            var holder = $('<div />').addClass('anura-asset-menu-holder').appendTo(menu);
            this.populateMenu(holder, asset);
        },
        populateMenu: function(holder, asset) {
            var _this = this;
            if (this.options.basket) {
                var link = $('<a />').addClass('anura-add-basket').attr('id', 'basket'+asset.id)
                    .attr('title', _this.strings['anura.addToBasket']).appendTo(holder).on('click', function() {
                        if (link.hasClass('added')) {
                            _this.options.basket.anuraBasket({remove: asset.id});
                            link.removeClass('added');
                        } else {
                            _this.options.basket.anuraBasket({add: asset.id, force_zip: _this.options.force_zip});
                            link.addClass('added');
                        }
                });
                $('<span />').addClass('anura-asset-menu-button anura-asset-menu-basket').appendTo(link);
                if ($.inArray(asset.id, this.options.basket.data('assets')) !== -1) link.addClass('added');
            }
            if (this.options.send) {
                var send = $('<a />').attr('rel', 'nofollow').appendTo(holder)
                    .attr('href', 'mailto:?body='+encodeURIComponent(_this.options.server+'/asset'+_this.options.ext
                        +'download='+asset.id+'&format='+this.options.send+'&locale='+_this.options.locale+(_this.options.force_zip?'&forcezip=true':'')));
                $('<span />').addClass('anura-asset-menu-button anura-asset-menu-mailto').appendTo(send);
            }
            if (this.options.download_directly) {
                $.each(this.download_formats, function(index, format) {
                    if (format.extensions && asset.extension && $.inArray(asset.extension, format.extensions) === -1) return;
                    var base = (format.url)?(format.url+'?'):('asset'+_this.options.ext+'format='+format.id+'&');
                    var link = $('<a />').attr('title', format.name).attr('target', '_blank').attr('rel', 'nofollow').appendTo(holder);
                    if (format.features && $.inArray('editable', format.features) !== -1) {
                        _this.element.anuraCropper({server: _this.options.server, locale: _this.options.locale, assetId: asset.id, downloadFormat: format.id, mode: 'download'});
                    } else {
                        link.attr('href', _this.options.server+'/'+base+'download='+asset.id+'&locale='+_this.options.locale+(_this.options.force_zip?'&forcezip=true':''))
                            .on('click', function () {$(window).trigger('anura-download', [asset.id, format.id]);});
                    }
                    $('<img />').addClass('anura-asset-menu-button').attr('src', _this.options.server+'/'+_this.options.icon_transform(format.icon)).appendTo(link);
                });
            }
        },
        bytesToSize: function(bytes) {
            var sizes = ['B&nbsp;', 'KB', 'MB', 'GB', 'TB', 'PB'];
            if (bytes === 0) return 'n/a';
            var magnitude = parseInt(Math.floor(Math.log(bytes) / Math.log(1024)));
            return (bytes / Math.pow(1024, magnitude)).toFixed(magnitude > 3 ? 2 : (magnitude < 2 ? 0 : 1))
                +'&thinsp;<span class="anura-filesize-unit">'+sizes[[magnitude]]+'</span>';
        },
        dateToString: function(date, type) {
            if (date === undefined) return "";
            switch (this.options.date_format) {
                case "iso":
                    var isoString = date.toISOString().split('.')[0];
                    if (type === 'date') return isoString.split('T')[0];
                    else return isoString.replace(/T/g, ' ');
                case "locale":
                    if (type === 'date') return date.toLocaleDateString(this.options.locale);
                    if (type === 'time') return date.toLocaleTimeString(this.options.locale).replace(/ ([AP])M/, '&nbsp;$1M');
                    else return date.toLocaleString(this.options.locale).replace(/ ([AP])M/, '&nbsp;$1M');
                default:
                    if (type === 'date') return $.anura.dateFormat(date, this.options.date_format.date ? this.options.date_format.date : 'yyyy-mm-dd');
                    if (type === 'time') return $.anura.dateFormat(date, this.options.date_format.time ? this.options.date_format.time : 'HH:MM');
                    else return $.anura.dateFormat(date, this.options.date_format.both ? this.options.date_format.both : 'yyyy-mm-dd HH:MM');
            }
        },
        shortenText: function(text) {
            if (!this.options.text_length_limit || !text || text.length < this.options.text_length_limit) return text;
            return text.slice(0, this.options.text_length_limit - 1)+'&hellip;'
        }
    };
    $.fn[pluginName] = function ( options ) {
        return this.each(function (i, item) { // iterate matches
            var instance = $.data(item, "plugin_" + pluginName);
            if (!instance) { // if it has no instance yet, create it
                $.data(item, "plugin_" + pluginName, new Plugin( item, options ));
            } else { // otherwise merge new options with existing ones, empty and re-init
                instance.options = $.extend( {}, instance.options, options );
                instance.init();
            }
        });
    };
})( jQuery, window, document );

/*!
 * jQuery appear plugin
 * Copyright (c) 2012 Andrey Sidorov, licensed under MIT license.
 * https://github.com/morr/jquery.appear, version: 0.3.6
 */
(function($){var selectors=[];var check_binded=false;var check_lock=false;var defaults={interval:250,force_process:false};var $window=$(window);var $prior_appeared=[];function process(){check_lock=false;for(var index=0,selectorsLength=selectors.length;index<selectorsLength;index++){var $appeared=$(selectors[index]).filter(function(){return $(this).is(':appeared')});$appeared.trigger('appear',[$appeared]);if($prior_appeared[index]){var $disappeared=$prior_appeared[index].not($appeared);$disappeared.trigger('disappear',[$disappeared])}$prior_appeared[index]=$appeared}};function add_selector(selector){selectors.push(selector);$prior_appeared.push()}$.expr[':']['appeared']=function(element){var $element=$(element);if(!$element.is(':visible')){return false}var window_left=$window.scrollLeft();var window_top=$window.scrollTop();var offset=$element.offset();var left=offset.left;var top=offset.top;if(top+$element.height()>=window_top&&top-($element.data('appear-top-offset')||0)<=window_top+$window.height()&&left+$element.width()>=window_left&&left-($element.data('appear-left-offset')||0)<=window_left+$window.width()){return true}else{return false}};$.fn.extend({appear:function(options){var opts=$.extend({},defaults,options||{});var selector=this.selector||this;if(!check_binded){var on_check=function(){if(check_lock){return}check_lock=true;setTimeout(process,opts.interval)};$(window).scroll(on_check).resize(on_check);check_binded=true}if(opts.force_process){setTimeout(process,opts.interval)}add_selector(selector);return $(selector)}});$.extend({force_appear:function(){if(check_binded){process();return true}return false}})})(function(){if(typeof module!=='undefined'){return require('jquery')}else{return jQuery}}());
