/* © brix cross media AG, http://www.brix.ch - It is strictly forbidden to change, reuse or copy the source code of this document. */
;(function ( $, window, document, undefined ) {

    var pluginName = "anuraTagCloud",
        defaults = {
            server: 'https://celum.local/anura/first', // REQUIRED, which anura endpoint to use
            ext: '.do?', // extension of server pages (.php, .do)
            node: {kind: 101, id: 0},
            sort: null, // custom sorting function(a, b) - see http://mzl.la/17qY6s9
            callback: function(server, kind, id, name) { // what happens when a node/leaf is clicked
                $('.anura-gallery-container').first().parent().anuraGallery({server: server, assets: null, node: {kind: kind, id: id}});
            },
            locale: '', // ISO 639-1 code, blank for default
            depth: 0, // recursion depth for lazy loading, 0 = no limit (default)
            font_size: {base: 75, multiplier: 100} // base size + (count/max * multiplier) in percent
        };

    function Plugin( element, options ) {
        this.element = this.container = $(element);
        this.options = $.extend( {}, defaults, options );
        this._defaults = defaults;
        this._name = pluginName;
        this.init();
    }

    Plugin.prototype = {
        init: function() {
            if (!this.options.node.id) return;
            var previous = this.element.find('.anura-tagcloud-container');
            if (previous.length > 0) this.container = previous.first();
            else this.container = $('<ul />').appendTo(this.element).addClass('anura-tagcloud-container');
            this.container.empty().addClass('anura-loading').removeClass(function (index, css) {return (css.match (/anura-locale-\S+/g) || []).join(' ');}).addClass('anura-locale-'+this.options.locale);
            if (!$.isFunction(this.options.sort)) this.sortKeywordToFunction();
            var _this = this;
            this.loadTree(_this.container, this.options.node.id, function(data) {
                _this.container.removeClass('anura-loading');
                _this.element.trigger('anura-loaded', [_this.options, data]);
            });
        },
        loadTree: function(target, id, internal_callback) {
            var _this = this;
            if (this.options.depth > 0) target.addClass('anura-loading');
            if (!$.isArray(id)) id = [id];
            $.anura.getJSON(this, this.options.server+'/node'+this.options.ext+'callback=?&tree='+id.join('&tree='), {
                kind: this.options.node.kind,
                depth: this.options.depth,
                locale: this.options.locale,
                asset_count: true
            }, 'anuraTagCloud', function(data) {
                var target = {flat: [], max: 0};
                if (data) _this.flattenTree(data, target, '');
                if ($.isFunction(_this.options.sort)) target.flat.sort(_this.options.sort);
                _this.populateCloud(target, _this.container);
                _this.container.find('a').on('click', function(e) {
                    e.preventDefault();
                    _this.options.callback(_this.options.server, _this.options.node.kind, $(this).data('id'), $(this).text());
                });
                _this.container.removeClass('anura-loading');
                _this.element.trigger('anura-loaded');
            });
        },
        flattenTree: function(data, target, path) {
            var _this = this;
            $.each(data, function(index, item) {
                if (item.assets > target.max) target.max = item.assets;
                item.path = path;
                target.flat.push(item);
                if (item.children && item.children.length > 0) {
                    _this.flattenTree(item.children, target, path +'/'+item.name);
                }
            });
        },
        populateCloud: function(data, container) {
            var _this = this;
            $.each(data.flat, function(index, item) {
                var li = $('<li />').addClass('anura-tag').appendTo(container);
                $('<a />').data('id', item.id).text(item.name).attr('title', item.path+'/'+item.name).appendTo(li)
                    .css('font-size', _this.options.font_size.base+((item.assets / data.max) * _this.options.font_size.multiplier)+'%');
            });
        },
        sortKeywordToFunction: function() {
            switch (this.options.sort) {
                case 'id_asc':        this.options.sort = function(a, b) {return a.id - b.id}; break;
                case 'id_desc':       this.options.sort = function(a, b) {return b.id - a.id}; break;
                case 'name_asc':      this.options.sort = function(a, b) {return a.name.localeCompare(b.name)}; break;
                case 'name_desc':     this.options.sort = function(a, b) {return b.name.localeCompare(a.name)}; break;
            }
        }
    };

    $.fn[pluginName] = function ( options ) {
        return this.each(function (i, item) { // iterate matches
            var instance = $.data(item, "plugin_" + pluginName);
            if (!instance) { // if it has no instance yet, create it
                $.data(item, "plugin_" + pluginName, new Plugin( item, options ));
            } else { // otherwise merge new options with existing ones, empty and re-init
                instance.options = $.extend( {}, instance.options, options );
                instance.init();
            }
        });
    };
})( jQuery, window, document );
