/* © brix cross media AG, http://www.brix.ch - It is strictly forbidden to change, reuse or copy the source code of this document. */
;(function ( $, window, document, undefined ) {

    var pluginName = "anuraTree",
        defaults = {
            server: 'https://celum.local/anura/first', // REQUIRED, which anura endpoint to use
            ext: '.do?', // extension of server pages (.php, .do)
            node: {kind: 101, id: 0},
            sort: 'name_asc', // custom sorting function(a, b) - see http://mzl.la/17qY6s9 or keyword id_asc, name_asc, info_asc (or _desc)
            callback: function(server, kind, id, name) { // what happens when a node/leaf is clicked
                $('.anura-gallery-container').first().parent().anuraGallery({server: server, assets: null, node: {kind: kind, id: id}});
            },
            locale: '', // ISO 639-1 code, blank for default
            depth: 0, // recursion depth for lazy loading, 0 = no limit (default)
            show: 0, // ID of the node to show on load
            list_style: true, // use list styles to indicate node/leaf/lazy
            li_expander: false, // enable expanding the tree by clicking on the <li> icon (not compatible with hiding the li icon or lazy loading)
            asset_count: false, // show how many assets are contained in this node (slow in 1.x!)
            hide_empty: false, // don't show nodes with no children, uses asset_count
            noderef: false, // count assets that have node referencing infofields to this node, 2.x
            show_root: false, // include the requested root node(s) in the tree
            show_tags: false, // show node icons (folders may have colored ones), 1.x
            classes: {container: '', firstul: '', ul: '', li: ''},
            state: '', // prefix for state in url (multi-instance) or false to disable state,
            data_modifier: null, // function to modify incoming tree data before it is rendered, i.e. function(data) {return data[0].name = 'first';}
            a_href_attr: null, // custom href attribute on hyperlinks, e.g. "javascript:" or "#" (the latter is incompatible with the state option)
            multiselect: false, // enables multiselect mode (probably only useful when used inside anuraSearch)
            infofields: [], // list of information field IDs to load with the tree response (use sparingly)
        };

    function Plugin( element, options ) {
        this.element = this.container = $(element);
        this.options = $.extend( {}, defaults, options );
        this.silentInit = false;
        this.deepLink = null;
        this._defaults = defaults;
        this._name = pluginName;
        this.init();
    }

    Plugin.prototype = {
        init: function() {
            if (!this.options.node.id) return;
            var previous = this.element.find('.anura-tree-container');
            if (previous.length > 0) this.container = previous.first();
            else this.container = $('<div />').appendTo(this.element).addClass('anura-tree-container');
            this.container.empty().addClass('anura-loading').addClass(this.options.classes.container)
                .removeClass(function (index, css) {return (css.match (/anura-locale-\S+/g) || []).join(' ');}).addClass('anura-locale-'+this.options.locale);
            if (this.options.show_tags) this.container.addClass('anura-tags');
            if (!this.options.list_style) this.container.addClass('no-list-style');
            if (!$.isFunction(this.options.sort)) this.sortKeywordToFunction();
            this.options.show = $.anura.getState(this.options.state, 'tree', this.options.show);
            this.silentInit = (this.options.state !== false && window.location.hash.indexOf('search-') > 0);
            if (this.options.depth > 0 && $.isNumeric(this.options.show)) this.deepLink = this.options.show; // support deep linking when lazy loading

            var _this = this;
            this.loadTree(_this.container, this.options.node.id, function(data) {
                $('.anura-tree-container ul ul').addClass('anura-tree-folded');
                _this.container.removeClass('anura-loading');
                _this.element.data('server', _this.options.server).data('kind', _this.options.node.kind).trigger('anura-loaded', [_this.options, data]);
                if (_this.options.show === 'first' || _this.options.show === true /*legacy*/) {
                    _this.container.find('li.anura-leaf a').first().click();
                } else if (_this.options.show === 'last') {
                    _this.container.find('li.anura-leaf a').last().click();
                } else if ($.type(_this.options.show) === 'string') {
                    $.each(_this.options.show.split(','), function (index, item) {
                        _this.container.find('#anura-node-'+item+' > a').trigger($.Event("click", { ctrlKey: true }));
                    });
                } else {
                    _this.container.find('#anura-node-'+_this.options.show+' > a').click();
                }
            }, true);
        },
        loadTree: function(target, id, internal_callback, first_time) {
            var _this = this;
            if (this.options.depth > 0) target.addClass('anura-loading');
            if (!$.isArray(id)) id = [id];

            var query = {
                kind: this.options.node.kind,
                depth: this.options.depth,
                locale: this.options.locale,
                asset_count: this.options.asset_count || this.options.hide_empty,
                with_root: this.options.show_root && first_time,
                noderef: this.options.noderef,
                deep_link: this.deepLink
            };
            if (this.options.infofields && this.options.infofields.length > 0) query.infofields = this.options.infofields.join(',');
            $.anura.getJSON(this, this.options.server+'/node'+this.options.ext+'callback=?&tree='+id.join('&tree='), query, 'anuraTree', function(data) {
                _this.deepLink = null; // reset after first time
                if (data && data.length === 0) {
                    // lazy loading has returned 0 children -> just mark as leaf
                    target.removeClass('anura-node').addClass('anura-leaf');
                } else {
                    var new_ul = $('<ul />').addClass(_this.options.classes.firstul).appendTo(target);
                    if (data) {
                        if ($.isFunction(_this.options.data_modifier)) _this.options.data_modifier(data);
                        _this.populateTree(data, new_ul);
                    }
                    new_ul.find('a').on('click', function(e) {
                        e.stopPropagation();
                        var _a = $(this), _li = _a.parent('li');
                        var name = _a.find('.anura-node-name').text();
                        if (_this.options.multiselect && (e.ctrlKey || e.metaKey || _this.element.data('selected') ==/*not ===*/ _a.data('id'))) {
                            _li.toggleClass('anura-current anura-active');
                        } else {
                            _this.container.find('li').removeClass('anura-active anura-current');
                            _li.addClass('anura-current anura-active');
                        }
                        _li.parents('li').addClass('anura-active');
                        _li.parents('ul').removeClass('anura-tree-folded').addClass('anura-tree-unfolded');
                        if (_this.options.multiselect) {
                            var selected = [], names = [];
                            _this.container.find('.anura-current > a').each(function (index, item) {
                                var node = $(item); selected.push(node.data('id')); names.push(node.data('names'));
                            });
                            _this.element.data('selected', selected).data('selected_name', names);
                        } else {
                            _this.element.data('selected', _a.data('id')).data('selected_name', name);
                        }
                        if (_this.options.li_expander !== true) _this.container.find('li').not('.anura-active').find('ul')
                            .removeClass('anura-tree-unfolded').addClass('anura-tree-folded');
                        _li.find('> ul').removeClass('anura-tree-folded').addClass('anura-tree-unfolded')
                            .find('> ul').removeClass('anura-tree-unfolded').addClass('anura-tree-folded');
                        var loaded = function() {
                            if (_this.silentInit) _this.silentInit = false;
                            else _this.options.callback(_this.options.server, _this.options.node.kind, _a.data('id'), name);
                            _this.element.trigger('anura-clicked', [_this.options.server, _this.options.node.kind, _a.data('id'), name]);
                        };
                        if (_li.hasClass('anura-lazy')) {
                            _li.removeClass('anura-lazy').addClass('anura-node');
                            _this.loadTree(_li, _a.data('id'), loaded, false);
                        } else {
                            loaded();
                        }
                        if (!_this.silentInit) $.anura.removeStates(_this.options.state, 'search-');
                        $.anura.pushState(_this.options.state, 'tree', selected?selected:_a.data('id'));
                    });
                }
                if (internal_callback) internal_callback(data);
                if (first_time) _this.element.trigger('anura-init', [_this.options, data]);
                else _this.element.trigger('anura-init-lazy', [_this.options, data]);
                target.removeClass('anura-loading');
            });
        },
        populateTree: function(data, target) {
            var _this = this;
            if ($.isFunction(this.options.sort)) data.sort(this.options.sort);
            $.each(data, function(index, item) {
                if (_this.options.hide_empty && item.assets < 1) return;
                var liId = 'anura-node-'+item.id, li = $('<li />').attr('id', liId).addClass(_this.options.classes.li).appendTo(target);
                if (item.infofield) li.data('infofield', item.infofield);
                if (item.infofields) $.each(item.infofields, function(infoId, infoVal) {li.data('node-infofield-'+infoId, infoVal);});
                if (item.infofields_raw) $.each(item.infofields_raw, function(infoId, infoVal) {li.data('node-infofield-raw-'+infoId, infoVal);});
                if (_this.options.li_expander) li.on('click', function(e) {
                    if ($(e.target).attr('id') === liId) {
                        var innerUl = li.find(' > ul');
                        if (innerUl.length === 0 || _this.options.li_expander === 'click') li.find('> a').click();
                        else li.find(' > ul').toggleClass('anura-tree-folded anura-tree-unfolded');
                    }
                });
                var a = $('<a/>').data('id', item.id).data('name', item.name).html('<span class="anura-node-name">'+item.name+'</span>').appendTo(li);
                if (_this.options.a_href_attr !== null) a.attr('href', _this.options.a_href_attr);
                if (_this.options.show_tags) {
                    var tag = (item.custom?item.custom:'!default')+'/small.png';
                    var tag_type = (_this.options.node.kind === 103?'keyword':(_this.options.node.kind === 102?'assetcollection':'folder'));
                    li.css('background-image', 'url("'+_this.options.server+'/../img/aci/'+tag_type+'/'+tag+'")');
                }
                if (_this.options.asset_count) a.html(a.html()+'&nbsp;<span class="anura-asset-count">'+(item.assets || 0)+'</span>');
                if (!item.children) {
                    li.addClass('anura-leaf');
                } else if (item.children.length > 0) {
                    li.addClass('anura-node');
                    _this.populateTree(item.children, $('<ul />').addClass(_this.options.classes.ul).appendTo(li));
                } else {
                    li.addClass('anura-lazy');
                }
            });
            if (_this.options.hide_empty && target.find('li').length === 0) { // if no children "survive", re-declare the node as a leaf
                target.parent().removeClass('anura-node').addClass('anura-leaf');
                target.remove();
            }
        },
        sortKeywordToFunction: function() {
            switch (this.options.sort) {
                case 'id_asc':        this.options.sort = function(a, b) {return a.id - b.id}; break;
                case 'id_desc':       this.options.sort = function(a, b) {return b.id - a.id}; break;
                case 'name_asc':      this.options.sort = function(a, b) {return a.name.localeCompare(b.name, undefined, {numeric: true, sensitivity: 'base'})}; break;
                case 'name_desc':     this.options.sort = function(a, b) {return b.name.localeCompare(a.name, undefined, {numeric: true, sensitivity: 'base'})}; break;
                case 'info_asc':      this.options.sort = function(a, b) {return ((a.infofield?a.infofield:a.name)).localeCompare((b.infofield?b.infofield:b.name), undefined, {numeric: true, sensitivity: 'base'})}; break;
                case 'info_desc':     this.options.sort = function(a, b) {return ((b.infofield?b.infofield:b.name)).localeCompare((a.infofield?a.infofield:a.name), undefined, {numeric: true, sensitivity: 'base'})}; break;
            }
        }
    };

    $.fn[pluginName] = function ( options ) {
        return this.each(function (i, item) { // iterate matches
            var instance = $.data(item, "plugin_" + pluginName);
            if (!instance) { // if it has no instance yet, create it
                $.data(item, "plugin_" + pluginName, new Plugin( item, options ));
            } else { // otherwise merge new options with existing ones, empty and re-init
                instance.options = $.extend( {}, instance.options, options );
                instance.init();
            }
        });
    };
})( jQuery, window, document );
